/*
 * a0-new-thr.c : creates many threads
 *
 * by Marcelo Pasin
 * Apache Project, IMAG, Grenoble, France
 * Jul 10, 1997
 *
 * This program checks how many threads can be created with ath0b.
 *
 * This program can run in any number of nodes, but one is enough.
 * If an argument is passed, it is used as the maximum number of
 * threads created. If no argument is passed, 1000 is the maximum.
 *
 * It starts and waits for the end of many consecutive threads. This
 * program was originally written to verify whether the POSIX library
 * had a bug or not, being able to create infinite consecutive threads.
 * The option A0_REUSETHR was inserted in the code of a0thread.c, in
 * order to reuse the threads instead of destroying them at every
 * a0ThreadExit().
 */

#include <a0b.h>
#include <stdio.h>

akpthread_t prev;
int count_new = 0;


a0tError my_thread(void *input)
{
	akpthread_t new;
	unsigned char *p;
	int n;
	char string[80];

	new = akpthread_self();
	if (!akpthread_equal(new, prev))
		count_new++;
	prev = new;

	a0ExitThread(0);
}


int main(int argc, char *argv[])
{
	void *rep;
	a0tThread slave;
	int max;
	int ii;

	a0Init(&argc, &argv);
	a0InitCommit();

	if (argv[1])
		max = atoi(argv[1]);
	else
		max = 1000;

	printf("MAX %d\n", A0MaxFifoPriority);

	prev = akpthread_self();

	for (ii = 0; ii < max;) {
		a0NewSlave(&slave, A0DefaultScheduling, A0DefaultPriority,
			   A0DefaultStack, my_thread, 0);
		a0JoinSlave(&slave, &rep);
		ii ++;
		if (!(ii % 100))
			printf("done %d threads, %d were new.\n",
				ii, count_new);
	}
	a0Terminate();
}
