/*
 * a0-srv-buf.c
 * Tests services and buffers
 *
 * Ilan, Jun 3, 1996.
 *
 * Needs no parameters. Must run on two nodes.
 *
 */

#include <a0b.h>
#include <stdio.h>


/*************************************************************************/

/* Global ports, services */
a0tPort port;
int serv_num, req_num;


/*************************************************************************/

showstats(char *khdfjhdbflsgjh)
{

	unsigned long id;

	_a0GetSelfThreadId(&id);

	/* these are undocumented variables: never used it! */
	printf("thread:%ld, %s: ", id, khdfjhdbflsgjh);
	printf("a0ThreadsTotal=%d, a0ThreadsBlockedComm=%d, "
	       "a0ThreadsBlockedLock=%d\n",
	       a0ThreadsTotal, a0ThreadsBlockedComm,
	       a0ThreadsBlockedLock);
}


/*************************************************************************/

void slave_code(void *param)
{
	a0tBuffer buff;
	a0tRequest req;
	int x;
	long serial;

	printf("Slave thread started. Parameter = %d\n", (int) param);
	showstats("Slave");

	a0NewBuffer(&buff, A0SendBufferType, 1000);

	/* Let's pack something into the buffer */
	x = (int) param;
	a0Pack(&buff, a0_INT, &x, 1);

	printf("After a0Pack\n");

	/* We send the buffer to node 1 */
	a0SendBuffer(&port, 1, 0, &req, &buff);
	_a0GetRequestSerialNo(&req, &serial);
	printf("Slave send: serial=%ld\n", serial);

	printf("After a0SendBuffer. Slave thread terminating\n");
}


/*************************************************************************/

a0tError req_func(a0tBuffer *b)
{

	int x;

	printf("Request starting\n");
	a0Unpack(b, a0_INT, &x, 1);
	printf("Value received: %d\n", x);
}


/*************************************************************************/

a0tError serv_func(a0tBuffer *mydata)
{

	int x;
	a0tBuffer buffer;
	a0tRequest req;
	a0tPort myport;
	long serial;

	printf("serv_func started on node %d\n", akNode);

	showstats("service");

	/* Get the node to answer on */
	a0Unpack(mydata, a0_PORT, &myport, 1);
	a0Unpack(mydata, a0_INT, &x, 1);
	/* Send a message to creator on node 0 so he's happy */
	a0NewBuffer(&buffer, A0SendBufferType, 100);
	x = 3 * x;
	a0Pack(&buffer, a0_INT, &x, 1);
	a0SendBuffer(&myport, 0, 0, &req, &buffer);

	_a0GetRequestSerialNo(&req, &serial);
	printf("serv_func send: serial=%ld\n", serial);
}


/*************************************************************************/

main(int argc, char **argv)
{
	a0tBuffer buffer, buffer2, buffer3;
	a0tRequest request;

	int x, rc;
	unsigned long bidon, sauv;
	int prio;
	a0tScheduling sched;

	a0tThread thread;
	a0tPort port2;
	void *result;

	/* Here we go... */
	a0Init(&argc, &argv);
	a0NewService(&serv_num, serv_func,
		 A0DefaultScheduling, A0DefaultPriority, A0DefaultStack);
	a0NewRRService(&req_num, req_func);
	a0InitCommit();

	showstats("main");

	a0GetSelfThreadPriority(&prio);
	a0GetSelfThreadScheduling(&sched);
	printf("main: prio=%d, sched=%d\n", prio, sched);

	if (akSize < 2) {
		printf("Pas assez de noeuds\n");
		exit(-1);
	}

	/* Creating port globally */
	a0NewPort(&port, 1, A0GlobalCreation);

	printf("After port creation\n");

	/* Node 0 sends to node 1 */
	if (akNode == 0) {

		/* A thread does a send to node 1 */
		rc = a0NewSlave(&thread, A0InheritScheduling,
				A0DefaultPriority, A0DefaultStack,
				(a0tFunction) slave_code,
				(void *) 1968);
		printf("return code of a0NewSlave=%d\n", rc);

		_a0GetThreadId(&thread, &bidon);
		printf("Slave just created: logical id=%ld\n", bidon);

		/* Create a remote thread on node 1 and wait for that
		 * thread to send back a message on port 2
		 */
		/* 1000 bytes buffer */
		a0NewBuffer(&buffer2, A0ThreadBufferType, 1000);
		a0NewPort(&port2, 1, TRUE);
		a0Pack(&buffer2, a0_PORT, &port2, 1);	/* Port the thread should use */
		x = 145;
		printf("Sending %x, should receive %d\n", x, 3 * x);
		a0Pack(&buffer2, a0_INT, &x, 1);

		showstats("main");

		a0StartRemoteThread(1, serv_num,
				    A0DefaultScheduling, A0DefaultPriority,
				    A0DefaultStack, &request, &buffer2);
		showstats("main");

		a0ReceiveBuffer(&port2, 1, 0, &request, &buffer2);
		x = 56;

		/* We unpack the int from the buffer */
		a0Unpack(&buffer2, a0_INT, &x, 1);
		printf("Received %d\n", x);
		a0NewBuffer(&buffer3, A0UrgentBufferType, 1000);
		x = 666;
		a0Pack(&buffer2, a0_INT, &x, 1);
		a0StartRemoteUrgent(1, req_num, &request, &buffer2);
		a0StartRemoteUrgent(0, req_num, &request, &buffer2);

		showstats("main");

		/* wait the slave to end */
		a0JoinSlave(&thread, &result);

	} else
	if (akNode == 1) {
		/* 1000 bytes buffer */
		a0NewBuffer(&buffer, A0SendBufferType, 1000);
		printf("After NewBuffer\n");
		showstats("main");

		showstats("main");

		/* We receive a buffer from node 0 */
		a0ReceiveBuffer(&port, 0, 0, &request, &buffer);

		showstats("main");

		printf("After a0ReceiveBuffer\n");

		x = 1996;
		/* We unpack the int from the buffer */
		a0Unpack(&buffer, a0_INT, &x, 1);

		printf("x received=%d\n", x);
	}

	showstats("main");
	printf("main before a0Terminate %d\n", a0SelfNode);

	a0Terminate();
}
