/*
 * a0f-dma-rw.c : many read/writes of DMA regions
 *
 * by Marcelo Pasin
 * Apache Project, IMAG, Grenoble, France
 * Jun 15, 1997
 *
 * This program tests read and write of ath0b DMA regions
 *
 * This program can run in any number of nodes. If an argument is
 * passed, it is the number of read/writes performed by the nodes.
 *
 * Node 0 creates 10 DMA regions and sends their IDs to all other nodes.
 * Every node (instead of 0) receives 10 DMA region IDs and stars a
 * loop. In the loop, a part of a randomly chosen DMA region is read and
 * echoed to stdin. A string is chosen at random at is written to the
 * same region just read.
 */

#include <a0format.h>
#include <stdio.h>


typedef char string[10];

a0tPort myport;


void dump(unsigned char *p, int s)
{
	char buf[1000];
	int i;

	for (i = 0; i < s; i++)
		sprintf(buf + 3 * i, "%02X ", p[i]);
	printf("%s\n", buf);
}


int main(int argc, char *argv[])
{
	a0tDMARegion glo[10];
	string variab[10];
	string values[25] =
	{
	"thread", "scheme", "done", "kernel", "tags", "used", "identify",
	    "sending", "and/or", "receive", "impose", "that", "relation",
	"several", "messages", "same", "have", "more", "than", "pending",
		"means", "decides", "which", "satisfy", "first"};

	int i, tot, n, g;
	a0tRequest req;
	string o;

	a0Init(&argc, &argv);
	a0UseFormats();
	a0NewPort(&myport, 1, 0);
	a0InitCommit();

	/* init a different random seed each time */
	srand(a0SelfNode * time(NULL));

	if (!a0SelfNode) {
		/* store random values in variab */
		for (i = 0; i < 10; i++) {
			strcpy(variab[i], values[rand() % 25]);
			printf("%s ", variab[i]);
		}
		printf("\n");

		/* create 10 DMA regions */
		for (i = 0; i < 10; i++)
			a0NewDMARegion(glo + i, A0FChar, variab[i], 10);

		/* tell everybody about all 10 DMA regions */
		for (i = 1; i < a0NodeCount; i++) {
			a0FSend(&myport, i, 0, &req, A0FDMARegion, glo, 10);
		}

		/* wait everybody end */
		for (i = 1; i < a0NodeCount; i++)
			a0FReceive(&myport, i, 0, &req, A0FInt, &n, 1);
		for (i = 0; i < 100; i++)
			a0YieldThread();
	} else {
		/* get all 10 DMA regions from master */
		a0FReceive(&myport, 0, 0, &req, A0FDMARegion, glo, 10);

		if (argc > 1)
			tot = atoi(argv[1]);
		else
			tot = 10;

		for (i = 0; i < tot; i++) {
			/* chose randomly a number and a region */
			n = (rand() & 0x7fff) % 25;
			g = (rand() & 0x7fff) % 10;
			a0FReadPart(glo + g, 1, 3, &req, A0FByte, o, 3);
			a0FWrite(glo + g, &req, A0FByte, values + n, 10);
			printf("%d glo[%d] get=%3.3s, put=%s\n", i, g, o, values + n);
		}

		/* tell 0 to die */
		a0FSend(&myport, 0, 0, &req, A0FInt, &n, 1);
	}
	a0Terminate();
}
