/*
 * a0samp7.c
 * Multithreaded ping-pong:
 * - Tracing average time for a ping-pong, function of the
 *       total number of threads.
 * - Average time for a thread to be scheduled again, function of the
 *       total number of threads
 *
 * Ilan, Jul 7, 1996.
 *
 */

#include <stdio.h>
#include "/usr/local/include/getrealtime.h"
#include <a0b.h>

/*************************************************************************/

/* Node where the pongs live */
#define PONG_NODE 1
/* Node where the pings live */
#define	PING_NODE 0

/*************************************************************************/

/* Maximum number of ping (pong) threads on PING_NODE (PONG_NODE)
 * (We're limited by the max number of threads our thread package can handle)
 */
#define	MAXPLAYERS 2000

/* Global ports */
a0tPort pong_port[MAXPLAYERS]; /* Ports for sending from ping to pong */
a0tPort ping_port[MAXPLAYERS]; /* Ports for sending from pong to ping */

a0tThread threads[MAXPLAYERS];

/* Results */
double t0s[MAXPLAYERS];
double t1s[MAXPLAYERS];

/* How many ping-pongs are done by each thread */
int loopsize;

/*************************************************************************/

showstats(char *khdfjhdbflsgjh)
{
  long id;

  a0GetSelfThreadId(&id);

  printf("thread:%ld, %s: ", id, khdfjhdbflsgjh);
  printf(
    "a0ThreadsTotal=%d, a0ThreadsBlockedComm=%d, a0ThreadsBlockedLock=%d\n",
  	a0ThreadsTotal, a0ThreadsBlockedComm, a0ThreadsBlockedLock);
}

/*************************************************************************/
/* Ping thread. param is the number of the port the ping must use */
ping(void *param)
{
  a0tBuffer buff;
  a0tRequest req;
  int i, rank;
  int bidon;
  long serial;
  double t0, t1;

  rank=(int) param;

  /* Send a first message to pong to signal him we're here */
  a0Send(pong_port+rank, PONG_NODE, 0, &req, a0_INT, &bidon, 0);
  /* Wait for pong to be ready */
  a0Receive(ping_port+rank, PONG_NODE, 0, &req, a0_INT, &bidon, 0);

  /* Ok, let's go */
  t0=getrealtime();
  for (i=0; i<loopsize; i++) {
    a0Send(pong_port+rank, PONG_NODE, 0, &req, a0_INT, &bidon, 0);
    a0Receive(ping_port+rank, PONG_NODE, 0, &req, a0_INT, &bidon, 0);
  }
  t1=getrealtime();

  /* Store the result for main thread to print it */
  t0s[rank]=t0;
  t1s[rank]=t1;

  /* We're done */
}

/*************************************************************************/
/* sched thread. param is the rank of the thread (to store results) */
sched(void *param)
{
  double t0, t1;
  int i, rank;

  rank=(int) param;

  /* Ok, let's go */
  t0=getrealtime();
  for (i=0; i<loopsize; i++) {
    a0YieldThread();
  }
  t1=getrealtime();

  /* Store the result for main thread to print it */
  t0s[rank]=t0;
  t1s[rank]=t1;

  /* We're done */
}

/*************************************************************************/
/* Pong thread. param is the number of the port the pong must use */
pong(void *param)
{
  a0tBuffer buff;
  a0tRequest req;
  int i, rank;
  int bidon;
  long serial;
  double t0, t1;

  rank=(int) param;

  /* Wait for ping to be ready */
  a0Receive(pong_port+rank, PING_NODE, 0, &req, a0_INT, &bidon, 0);
  /* Send a message to ping to signal him we're here */
  a0Send(ping_port+rank, PING_NODE, 0, &req, a0_INT, &bidon, 0);

  /* Ok, let's go */
  t0=getrealtime();
  for (i=0; i<loopsize; i++) {
    a0Receive(pong_port+rank, PING_NODE, 0, &req, a0_INT, &bidon, 0);
    a0Send(ping_port+rank, PING_NODE, 0, &req, a0_INT, &bidon, 0);
  }
  t1=getrealtime();

  /* Store the result for main thread to print it */
  t0s[rank]=t0;
  t1s[rank]=t1;

  /* We're done */
}

/*************************************************************************/

/* vars for ping_pong() and scheduling() */

int party_size; /* How many threads will play */
char *filename;

/*************************************************************************/

ping_pong()
{
  void *result; /* Not used, but needed */
  double sum, temp, average;
  FILE *file;
  int i;

  /* Creating all the ports */
  for (i=0; i<party_size; i++) {
    a0NewPort(pong_port+i, 1, FALSE);
    a0NewPort(ping_port+i, 1, FALSE);
  }

  /* Node 0 sends to node 1 */
  if (akNode==PING_NODE) {
    /* Create all the ping threads */
    for (i=0; i<party_size; i++) {
      a0NewSlave(threads+i, a0DefaultScheduling, a0DefaultPriority,
      		a0DefaultStack, ping, i);
    }

    /* Wait for all the pings to finish */
    for (i=0; i<party_size; i++) a0JoinSlave(threads+i, &result);

    /* We now print the times, and the average (until we do better) */
    sum=0;
    for (i=0; i<party_size; i++) {
      temp=t1s[i]-t0s[i];
      /* printf("Ping %d: %f sec. (-> %f)\n", i, temp, temp/loopsize); */
      sum += t1s[i]-t0s[i];
    }

    /* Average */
    average=sum/party_size;
    printf("Ping: %f sec (%d threads, %d r.t.)\n", average/loopsize,
    			party_size, loopsize);
    file=fopen(filename, "a");
    fprintf(file, "# ping trace. loopsize=%d\n", loopsize);
    fprintf(file, "%d %f\n", party_size, average/loopsize);
    fclose(file);
  }
  else if (akNode==PONG_NODE) {
    /* Create all the pong threads */
    /* for (i=0; i<party_size; i++) { */
    /* Let's create the threads in another order, to see what happens */
    for (i=party_size-1; i>=0; i--) {
      a0NewSlave(threads+i, a0DefaultScheduling, a0DefaultPriority,
      		a0DefaultStack, pong, i);
    }

    /* Wait for all the pongs to finish */
    for (i=0; i<party_size; i++) a0JoinSlave(threads+i, &result);

    /* We now print the times, and the average (until we do better) */
    sum=0;
    for (i=0; i<party_size; i++) {
      temp=t1s[i]-t0s[i];
      /* printf("Pong %d: %f sec. (-> %f)\n", i, temp, temp/loopsize); */
      sum += t1s[i]-t0s[i];
    }

    /* Average */
    average=sum/party_size;
    printf("Pong: %f sec (%d threads, %d r.t.)\n", average/loopsize,
    			party_size, loopsize);
  }

}

/*************************************************************************/

scheduling()
{
  int i;
  void *result; /* Not used, but needed */
  double sum, temp, average;
  FILE *file;

  /* We run with only one node now */
  /* Creating threads on the node */

  for (i=0; i<party_size; i++) {
      a0NewSlave(threads+i, a0DefaultScheduling, a0DefaultPriority,
      		a0DefaultStack, sched, i);
    }

    /* Wait for all the pings to finish */
    for (i=0; i<party_size; i++) a0JoinSlave(threads+i, &result);

    /* We now print the times, and the average (until we do better) */
    sum=0;
    for (i=0; i<party_size; i++) {
      temp=t1s[i]-t0s[i];
      /* printf("Scheduling %d: %f sec. (-> %f)\n", i, temp, temp/loopsize); */
      sum += t1s[i]-t0s[i];
    }

    /* Average */
    average=sum/party_size;
    printf("Sched: %f sec (%d threads, %d r.t.)\n", average/loopsize,
    			party_size, loopsize);
    file=fopen(filename, "a");
    fprintf(file, "# Scheduling trace. loopsize=%d\n", loopsize);
    fprintf(file, "%d %f\n", party_size, average/loopsize);
    fclose(file);
}

/*************************************************************************/
usage()
{
  if (akNode) return;
           printf("usage: poe a0samp7 <MPI parameters> <t> <party size>");
/* argv */ printf("<loopsize> <file>\n");
 /* 1 */   printf("t: 1: ping pong 2: thread scheduling time\n");
 /* 2 */   printf("party size: number of threads\n");
 /* 3 */   printf("loopsize: number of ping-pongs to average\n");
 /* 4 */   printf("file: file to append the trace to (use /dev/null...)\n");
}

/*************************************************************************/

main(int argc, char **argv)
{

  int i, type;


  /* Here we go... */
  a0Init(&argc, &argv);
  a0InitCommit();

  if (argc!=5) {
	   usage();
           a0Abort(1);
  }

  type=atoi(argv[1]);
  party_size=atoi(argv[2]);
  loopsize=atoi(argv[3]);
  filename=argv[4];
  if ((type!=1 && type!=2) || party_size<1 || loopsize <1) {
    usage();
    a0Abort(1);
  }

  if (type==1 && akSize != 2) {
    printf("Exactely 2 nodes needed for a ping pong\n");
    a0Abort(1);
  }

  if (type==2 && akSize !=1) {
    printf("Exactely 1 node needed for scheduling trace\n");
    a0Abort(1);
  }

  if (!akNode) {
    printf("%d threads, loopsize=%d\n", party_size, loopsize);
    printf("Output sent (append) to %s\n", filename);
  }


  if (type==1) ping_pong();
  else if (type==2) scheduling();
  a0Terminate();
}

