/*
 *
 * _a0buffer.h : Functions for the Ath0 buffers
 *
 * by Ilan Ginzburg, Marcelo Pasin
 * Apache Project, IMAG, Grenoble, France
 * May 28, 1996
 *
 */


/* Eliminates multiple includes of this file */
#ifndef _A0BUFFER_H
#define _A0BUFFER_H

#include <akernel.h>
#include "a0mpi.h"
#include "a0error.h"


/***************************************************************************/

typedef enum {
    A0NullBufferType = 0,
    A0SendBufferType,
    A0UrgentBufferType,
    A0ThreadBufferType,
    A0ReservedBufferType
  } a0tBufferType;

/***************************************************************************/

/* Structure referencing the actual storage space. The actual storage space
 * is composed of two parts:
 * - The first one for Athapascan data,
 * - The second one for user packed data.
 * When the buffer is created, Athapascan (dummy) data are first packed, then
 * the user packs his data, and just before the buffer is actually used, real
 * athapascan data are packed over the place where they were first packed.
 * Doesn't seem clean to you? got a better idea? I don't.
 */
typedef struct _a0tBuffer {
	a0tBufferType bufftype;	/* type of the buffer */
	void *storage;		/* actual storage space */
	size_t size;		/* total size of storage space */
	long position;		/* position in storage space */
	long userdata;		/* start of user data */
} a0tBuffer;

/***************************************************************************/


extern a0tError _a0NewBuffer(    a0tBuffer *buffer,
                                a0tBufferType bufftype,
                                size_t size);
extern a0tError _a0DisposeBuffer(a0tBuffer *buffer);
extern a0tError _a0ClearBuffer(a0tBuffer *buffer);

extern a0tError _a0Pack(a0tBuffer *buffer, a0tDatatype datatype,
			void *inbuf, int incount);
extern a0tError _a0Unpack(a0tBuffer *buffer, a0tDatatype datatype,
			void *outbuf, int outcount);

extern a0tError _a0GetPackedSize(a0tDatatype datatype, size_t *size);

/* backward compatibility, temporary defines */
#define A0TypeSendBuffer A0SendBufferType
#define A0TypeRRBuffer	A0UrgentBufferType
#define A0TypeRTCBuffer	A0ThreadBufferType
#define A0RRBufferType	A0UrgentBufferType
#define A0RTCBufferType A0ThreadBufferType

/* new stuff, with user controlled buffer size & position */
extern a0tError _a0NewBufferStorage(a0tBuffer *buffer,
			a0tBufferType bufftype,
			size_t size, void *storage);

#define _a0GetBufferPosition(buf, posp)	\
	(*(posp) = (buf)->position, A0ErrOk)

#define _a0SetBufferPosition(buf, pos) \
      (((pos >= 0) && (pos < (buf)->size)) ? \
  	      (((buf)->position = (pos)), A0ErrOk) : \
  	      A0ErrInvalid)


#ifdef A0_SOURCE
/* Defines only used for compiling Athapascan0 */

extern a0tError _a0AllocateBuffer(a0tBuffer **bufferpp,
				a0tBufferType bufftype, size_t size);
extern a0tError _a0DeallocateBuffer(a0tBuffer *bufferp);

#endif /* A0_SOURCE */


#endif /* _A0BUFFER_H */
