/*
 *
 * a0error.c : Error handling functions
 *
 *
 * by Marcelo Pasin
 * Apache Project, IMAG, Grenoble, France
 * Feb 16, 1996
 *
 */

#include <akernel.h>
#include <errno.h>
#include <stdio.h>

#define A0_SOURCE
#include "a0error.h"


/* error messages table */
static char *_a0InternalErrorMessage[] = {
	"internal error",
	"daemon msg",
	"memory violation",
	"daemon format",
	"bad service number"
};
static char *_a0FatalErrorMessage[] = {
	"fatal error",
	"mutex create",
	"mutex lock",
	"mutex unlock",
	"mutex destroy",
	"cond init",
	"cond signal",
	"cond wait",
	"cond destroy",
	"semap create",
	"semap P",
	"semap V",
	"semap destroy",
	"no memory",
	"comm dup",
	"comm rank",
	"comm type",
	"daemon thread create",
	"thread attr",
	"new format",
	"detach",
	"thread specific",
	"kernel communication",
};


/*************************************************************************
	_a0ErrorPrint - Print the errors in Athapascan
*************************************************************************/

extern a0tError _a0ErrorPrint(char *file, int line, a0tError err, char *msg) {
  if (!msg) msg = "no message";

  if (err<=A0ErrFatal) 
    fprintf(stderr, "ath0b fatal error (%s:%d): %s %s (errno=%d)\n",
    		file, line,
		_a0FatalErrorMessage[A0ErrFatal-err], msg, errno);
  else {
    /* if the error is not fatal, return to caller */
    fprintf(stderr, "ath0b error (%s:%d): %s %s (errno=%d)\n", file, line,
		_a0InternalErrorMessage[A0ErrInternal-err], msg, errno);
    return err;
  }

  /* blow up everything */
  /*  _a0Terminate(); */
  fflush(stderr);
  exit(err);
}


/*************************************************************************
	_a0KernelErrorConvert - convert akernel to _a0b errors
*************************************************************************/

extern a0tError _a0KernelErrorConvert(int kerror) {

  /* the kernel errors are translated to a slice of the _a0b errors */
  if (!kerror) return A0ErrOk;
  return kerror + A0ErrKernel;
}
