/*
 *
 * a0flock.h : a0 Functions of DMA lock
 *
 * by Marcelo Pasin
 * Apache Project, IMAG, Grenoble, France
 * Oct 18, 1996
 *
 */

/* Eliminates multiple includes of this file */
#ifndef _A0FLOCK_H
#define _A0FLOCK_H

#include <ath0.h>
#include "a0fmem.h"


/****************defines********************/
/****************types********************/

typedef struct {
  a0tDMARegion	*regionp;	/* region to lock */
  int		start;		/* start element */
  int		size;		/* number of elements */
  struct {
    a0tFormat	*formatp;	/* user format to read */
    void	*address;	/* user address to read */
    int		count;		/* user count to read */
  } read;
  struct {
    a0tFormat	*formatp;	/* user format to write */
    void	*address;	/* user address to write */
    int		count;		/* user count to write */
  } write;
  a0tMemoryAddress lockaddr;	/* remote lock address */
} a0tLock;


/****************variables********************/

/****************functions********************/

#define _a0SetLockRead(lock, i, f, a, c) ( \
			(lock)[i].read.formatp = f,	\
			(lock)[i].read.address = a,	\
			(lock)[i].read.count = c,	\
			A0ErrOk)

#define _a0SetUnlockWrite(lock, i, f, a, c) ( \
			(lock)[i].write.formatp = f,	\
			(lock)[i].write.address = a,	\
			(lock)[i].write.count = c,	\
			A0ErrOk)

#define _a0SetLockDMARegion(lock, i, regp) ( \
			(lock)[i].regionp = regp, \
			(lock)[i].start = 0, \
			(lock)[i].size = (regp)->size, \
			(lock)[i].read.formatp = NULL, \
			(lock)[i].write.formatp = NULL, \
			A0ErrOk)

#define _a0SetLockDMARegionPart(lock, i, regp, st, sz) ( \
			(lock)[i].regionp = regp, \
			(lock)[i].start = st, \
			(lock)[i].size = sz, \
			(lock)[i].read.formatp = NULL, \
			(lock)[i].write.formatp = NULL, \
			(((sz)+(st))>(regp)->size)?A0ErrInvalid:A0ErrOk)

a0tError _a0Lock(a0tLock *lock, int count, a0tRequest *requestp);
a0tError _a0Unlock(a0tLock *lock, int count, a0tRequest *requestp);
a0tError _a0ILock(a0tLock *lock, int count, a0tRequest *requestp);
a0tError _a0IUnlock(a0tLock *lock, int count, a0tRequest *requestp);

a0tError _a0IFReadPartLock(a0tLock *lock,
			a0tDMARegion *region, int start, int size,
			a0tRequest *reqp,
			a0tFormat *formatp, void *address, int count);
a0tError _a0FReadPartLock(a0tLock *lock,
			a0tDMARegion *region, int start, int size,
			a0tRequest *reqp,
			a0tFormat *formatp, void *address, int count);

a0tError _a0IFWriteUnlock(a0tLock *lock, a0tRequest *reqp,
			a0tFormat *formatp, void *address, int count);
a0tError _a0FWriteUnlock(a0tLock *lock, a0tRequest *reqp,
			a0tFormat *formatp, void *address, int count);

#define _a0IFReadLock(lock, rgn, req, fmt, ad, ct) \
	_a0IFReadPartLock(lock, rgn, 0, (rgn)->size, \
			req, fmt, ad, ct)

#define _a0FReadLock(lock, rgn, req, fmt, ad, ct) \
	_a0FReadPartLock(lock, rgn, 0, (rgn)->size, \
			req, fmt, ad, ct)


#ifdef A0F_SOURCE
/****************************************************************************
		Included only for compiling the lib
****************************************************************************/


a0tError _a0InitLocks(void);


#endif /* A0F_SOURCE */

#endif /* _A0FLOCK_H */
