/*
 *
 * a0fread.c : a0 read functions
 *
 * by Marcelo Pasin
 * Apache Project, IMAG, Grenoble, France
 * Feb 16, 1996
 *
 */

#define A0_SOURCE
#include <ath0.h>
#include <stdlib.h>

#define A0F_SOURCE
#include "a0fmem.h"
#include "a0fread.h"


#define a0Debug_on(a) printf a
#define a0Debug(a)


/******************local variables & definitions*****************/


/*************************************************************************
	_a0IFReadPart - perform a partial non-blocking DMA read
*************************************************************************/

extern a0tError _a0IFReadPart(a0tDMARegion *regionp, int start, int size,
			a0tRequest *reqp,
			a0tFormat *formatp, void *address, int count) {

  a0tMemRequest *message;
  a0tError r;
  int tag;

  if ((start+size) > regionp->size) return A0ErrInvalid;

  message = (a0tMemRequest *) malloc(sizeof(a0tMemRequest));
  if (!message) return A0ErrNoMem;
  _a0FindMemTag(&tag);

  message->type = A0ReadRequest;
  message->region = *regionp;
  message->region_start = start;
  message->region_size = size;
  message->reply_node = akNode;
  message->reply_tag = tag;

  r = _a0ISendCallback(&_a0MemServerPort, regionp->node, 0, NULL,
		a0_MEM_REQUEST, message, 1, free, message);
  if (r) return r;

  a0Debug(("post read req to %d format %x data %x start %d size %d\n",
	regionp->node,
	regionp->format.address, regionp->data.address, start, size));

  return _a0IFReceive(&_a0MemReplyPort, regionp->node, tag, reqp,
			formatp, address, count);
}


/*************************************************************************
	_a0FReadPart - perform a partial blocking DMA read
*************************************************************************/

extern a0tError _a0FReadPart(a0tDMARegion *regionp, int start, int size,
			a0tRequest *reqp,
			a0tFormat *formatp, void *address, int count) {

  a0tMemRequest message;
  a0tError r;
  int tag;

  if ((start+size) > regionp->size) return A0ErrInvalid;
  _a0FindMemTag(&tag);

  message.type = A0ReadRequest;
  message.region = *regionp;
  message.region_start = start;
  message.region_size = size;
  message.reply_node = akNode;
  message.reply_tag = tag;

  r = _a0ISend(&_a0MemServerPort, regionp->node, 0, NULL,
		a0_MEM_REQUEST, &message, 1);
  if (r) return r;

  a0Debug(("post read req to %d format %x data %x start %d size %d\n",
	regionp->node,
	regionp->format.address, regionp->data.address, start, size));

  return _a0FReceive(&_a0MemReplyPort, regionp->node, tag, reqp,
			formatp, address, count);
}


/*************************************************************************
	_a0ReplyRead
*************************************************************************/

a0tError _a0ReplyRead(a0tMemRequest *answerp) {

  void *address;

  /* calculate address */
  if (_a0GetFormatOffset(answerp->region.format.address,
			 answerp->region.data.address,\
			 answerp->region_start, &address))
    _a0Error(A0ErrDaemonMsg, "mem offset");

  a0Debug(("will send (%d:%d), fmt=%x, adr=%x, sz=%d\n",
	answerp->reply_node, answerp->reply_tag,
	answerp->region.format.address, address, answerp->region_size));

  if (_a0FSend(&_a0MemReplyPort,
			answerp->reply_node, answerp->reply_tag, NULL,
			answerp->region.format.address,
			address,
			answerp->region_size))
	_a0Error(A0ErrDaemonMsg, "read reply");
}
