/*
 *
 * a0fuser.h : _a0 Format user functions
 *
 * by Marcelo Pasin
 * Apache Project, IMAG, Grenoble, France
 * May 23, 1996
 *
 */


/* Eliminates multiple includes of this file */
#ifndef _A0FUSER_H
#define _A0FUSER_H

#include <ath0.h>


/****************types********************/

typedef enum {
  A0NullFormatType = 0,
  A0Map,
  A0Xdr,
  A0LastFormatType
} a0tFormatType;


typedef struct _a0tFormat {
  a0tFormatType type;
  union {
    struct _a0tMap *mapp;
    struct _a0tXdr *xdrp;
  } u;
} a0tFormat;


/****************variables********************/

extern a0tFormat _a0BasicFormats[];
extern int _a0UseFormatsRun;

#define A0FChar		(_a0BasicFormats+0)
#define A0FShort	(_a0BasicFormats+1)
#define A0FInt		(_a0BasicFormats+2)
#define A0FLong		(_a0BasicFormats+3)
#define A0FUChar	(_a0BasicFormats+4)
#define A0FUShort	(_a0BasicFormats+5)
#define A0FUInt		(_a0BasicFormats+6)
#define A0FULong	(_a0BasicFormats+7)
#define A0FFloat	(_a0BasicFormats+8)
#define A0FDouble	(_a0BasicFormats+9)
#define A0FLDouble	(_a0BasicFormats+10)
#define A0FByte		(_a0BasicFormats+11)
#define A0FDMARegion	(_a0BasicFormats+12)
#define A0FPort		(_a0BasicFormats+13)
#define A0FAddress	(_a0BasicFormats+14)


/****************functions********************/

extern a0tError _a0GetFormatOffset(a0tFormat *formatp, 
			void *address, int count,
			void **offset);
extern a0tError _a0GetFormatSize(a0tFormat *formatp, 
			void *address, int count, 
			size_t *sizep);
extern a0tError _a0GetFormatCount(a0tFormat *formatp, 
			void *address, int size, int *countp);


extern a0tError _a0UseFormats(void);

#ifdef PUT_IT_BACK_MARCELO
extern a0tError _a0NewFormat(a0tFormat *newformatp, 
				int formattype, ... );
#endif

/* Sends a formatted message (nonblocking). */
extern a0tError _a0IFSend(
		a0tPort *port, int dest, int usertag, 
		a0tRequest *req,
		a0tFormat *formatp, void *address, int count);
/* Sends a formatted message (blocking). */
extern a0tError _a0FSend(
		a0tPort *port, int dest, int usertag, 
		a0tRequest *req,
		a0tFormat *formatp, void *address, int count);
/* Receives a formatted message (nonblocking). */
extern a0tError _a0IFReceive(
		a0tPort *port, int src, int usertag, 
		a0tRequest *req,
		a0tFormat *formatp, void *address, int count);
/* Receives a formatted message (blocking). */
extern a0tError _a0FReceive(
		a0tPort *port, int src, int usertag, 
		a0tRequest *req,
		a0tFormat *formatp, void *address, int count);

/* formatted packing & unpacking */
extern a0tError _a0FPack(a0tBuffer *buffer,
			a0tFormat *format, void *address, int count);
extern a0tError _a0FUnpack(a0tBuffer *buffer,
			  a0tFormat *format, void *address, int count);

extern a0tError _a0IFSendCallback(
                a0tPort *port, int dest, int tag, a0tRequest *req,
                a0tFormat *formatp, void *address, int count,
                void (*callback)(void *), void *callback_par);

extern a0tError _a0IFReceiveCallback(
                a0tPort *port, int src, int tag, a0tRequest *req,
                a0tFormat *formatp, void *address, int count,             
                void (*callback)(void *), void *callback_par);


#ifdef A0F_SOURCE
/****************************************************************************
		Included only for compiling the lib
****************************************************************************/

extern a0tError _a0SetBasicFormat(a0tFormat *formatp,
					a0tDatatype datatype);

#endif /* A0F_SOURCE */

#endif /* _A0FUSER_H */
