/*
 *
 * akconfig.h: akernel configuration switches
 *
 * by Marcelo Pasin
 * Apache Project, IMAG, Grenoble, France
 * Jun 13, 1997
 *
 */

/* Eliminates multiple includes of this file */
#ifndef _AKCONFIG_H
#define _AKCONFIG_H

/* This file contains all configuration definitions for akernel. The
   general behavior of akdaemon and akcomm, for every achitecture
   supported, is expressed here. */

/******************************************************************
 * Set one or more of the following, to choose the daemon policy  *
 ******************************************************************
AK_ASLEEP - the daemon always sleeps in a condition variable. If not set,
	the daemon sleeps when the queue is empty or yields when it is
	not empty and all requests were tested.
AK_WAKER - run a low-priority thread that wakes up the daemon thread
	when no other threads run. Must have priorities in pthreads.
AK_TIMED - the daemon sleeps with a timeout instead of yielding. Used if
	AK_ASLEEP is not defined. akpthread_cond_timedwait must work.
AK_NOYLD - the daemon does not call sched_yield() after having tested
	all requests. It just unlocks then locks back the mutex. It 
	avoids too much context changes. Used if AK_ASLEEP and AK_TIMED
	are not	defined.
AK_NSTOP - the daemon runs non-stop if awaken by the waker-up thread.
	Used with AK_WAKER. Should be always used if AK_WAKER is used
	and akpthread_cond_timedwait doesn't work, because there is no
	reason to stop the daemon if no other threads run.
AK_CANCEL - daemon should cancel all MPI pending requests before it
	dies. Used with the MPIs that complain about that.
AK_DAEMON - daemon sould run. If not defined, the daemon doesn't run
	at all.

 ********************************************************************
 * Set the priorities and scheduling according to the daemon policy *
 ********************************************************************
AK_DAEMON_SLEEPDELTA - delay to sleep the daemon, in nanoseconds. Used
	with AK_TIMED.
AK_DAEMON_SCHED and AK_DAEMON_PRIO - scheduling and priority to be used
	with the kernel daemon.
AK_WAKER_SCHED and AK_WAKER_PRIO - scheduling and priority to be used
	with the kernal daemon waker.
AK_USER_SCHED and AK_USER_PRIO - default scheduling and priority of the
	user threads.

 ********************************************************************
 * akcomm fine tuning						    *
 ********************************************************************

AK_BLKMPI - threads block directly inside MPI. It presuposes that MPI
	is thread-safe.
AK_DAENUL - The threads don't pass the w.c. to the daemon, because they 
	will block in MPI directly. The akDaemon is only needed for the
	NULL handles and for akWaitAny. Must be used with AK_BLKMPI.
*/


#if defined(A0_LAM) || defined(A0_MPICH)
#define AK_CANCEL
#endif

#if defined(A0_TAIX3)
/*#if defined(A0_MPIF)*/
#define AK_ASLEEP
#define AK_NSTOP
#define AK_WAKER
#define AK_WAKER_SCHED	akSCHED_RR
#define AK_WAKER_PRIO	akSCHED_PRI_RR_MIN_NP
/*#endif*/
#define AK_DAEMON
#define AK_DAEMON_SCHED	akSCHED_FIFO
#define AK_DAEMON_PRIO	akSCHED_PRI_FIFO_MAX_NP
#define AK_USER_SCHED	akSCHED_RR
#define AK_USER_PRIO	((akSCHED_PRI_RR_MAX_NP+akSCHED_PRI_RR_MIN_NP)/2)
#endif

#if defined(A0_TSOL2)
#define AK_DAEMON
#define AK_DAEMON_SCHED	akSCHED_OTHER
#define AK_DAEMON_PRIO	((akSCHED_PRI_OTHER_MAX_NP+akSCHED_PRI_OTHER_MIN_NP)/2)
#define AK_USER_SCHED	akSCHED_OTHER
#define AK_USER_PRIO	((akSCHED_PRI_OTHER_MAX_NP+akSCHED_PRI_OTHER_MIN_NP)/2)
#endif

#if defined(A0_TLIN2)
#define AK_DAEMON
#define AK_DAEMON_SCHED	akSCHED_OTHER
#define AK_DAEMON_PRIO	((akSCHED_PRI_OTHER_MAX_NP+akSCHED_PRI_OTHER_MIN_NP)/2)
#define AK_USER_SCHED	akSCHED_OTHER
#define AK_USER_PRIO	((akSCHED_PRI_OTHER_MAX_NP+akSCHED_PRI_OTHER_MIN_NP)/2)
#endif


#if defined(A0_TAIX4)
#if defined(A0_MPIBM)
/*#define AK_BLKMPI*/
/*#define AK_DAENUL*/
#endif
#define AK_DAEMON
#define AK_DAEMON_SCHED	akSCHED_OTHER
#define AK_DAEMON_PRIO	akSCHED_PRI_OTHER_MIN_NP
#define AK_USER_SCHED	akSCHED_OTHER
#define AK_USER_PRIO	akSCHED_PRI_OTHER_MIN_NP
#endif

#if defined(A0_THP10)
#define AK_DAEMON
#define AK_WAKER
#define AK_ASLEEP
#define AK_DAEMON_SCHED	akSCHED_FIFO
#define AK_DAEMON_PRIO	akSCHED_PRI_FIFO_MAX_NP
#define AK_WAKER_SCHED	akSCHED_RR
#define AK_WAKER_PRIO	akSCHED_PRI_RR_MIN_NP
#define AK_USER_SCHED	akSCHED_RR
#define AK_USER_PRIO	((akSCHED_PRI_RR_MAX_NP+akSCHED_PRI_RR_MIN_NP)/2)
#endif

#if defined(A0_TOSF4)
#define AK_DAEMON
#define AK_TIMED
#define AK_DAEMON_SCHED	akSCHED_FIFO
#define AK_DAEMON_PRIO	akSCHED_PRI_FIFO_MAX_NP
#define AK_DAEMON_SLEEPDELTA 100000
#define AK_USER_SCHED	akSCHED_RR
#define AK_USER_PRIO	((akSCHED_PRI_RR_MAX_NP+akSCHED_PRI_RR_MIN_NP)/2)
#endif



/* New IRIX6 configuration. Done after Pierre Eric's problems, i.e.,
 * starvation on some threads and daemon thread running so much
 */
 
#if defined(A0_TSGI6)
#define AK_DAEMON
#define AK_DAEMON_SCHED	akSCHED_OTHER
#define AK_DAEMON_PRIO	((akSCHED_PRI_OTHER_MAX_NP+akSCHED_PRI_OTHER_MIN_NP)/2)
#define AK_USER_SCHED	akSCHED_OTHER
#define AK_USER_PRIO	((akSCHED_PRI_OTHER_MAX_NP+akSCHED_PRI_OTHER_MIN_NP)/2)
#endif

#ifdef A0_TMAR
#define AK_DAEMON
#define AK_DAEMON_SCHED	akSCHED_OTHER
#define AK_DAEMON_PRIO	akSCHED_PRI_OTHER_MAX_NP
#define AK_USER_SCHED	akSCHED_OTHER
#define AK_USER_PRIO	((akSCHED_PRI_OTHER_MAX_NP+akSCHED_PRI_OTHER_MIN_NP)/2)
#endif

#if defined(A0_TMIT)
#define AK_DAEMON
#define AK_WAKER
#define AK_ASLEEP
#define AK_DAEMON_SCHED	akSCHED_FIFO
#define AK_DAEMON_PRIO	akSCHED_PRI_FIFO_MIN_NP
#define AK_WAKER_SCHED	akSCHED_FIFO
#define AK_WAKER_PRIO	akSCHED_PRI_RR_MIN_NP
#define AK_USER_SCHED	akSCHED_FIFO
#define AK_USER_PRIO	akSCHED_PRI_FIFO_MAX_NP
/*#define AK_USER_PRIO	((akSCHED_PRI_RR_MAX_NP+akSCHED_PRI_RR_MIN_NP)/2)*/
#endif


#endif /* _AKCONFIG_H */
