/*
 *
 * akerror.c : Error handling functions
 *
 *
 * by Marcelo Pasin, Ilan Ginzburg
 * Apache Project, IMAG, Grenoble, France
 * Feb 5, 1996
 *
 */

#include <pthread.h>
#include <stdio.h>
#include <errno.h>

#define AKERNEL_SOURCE
#include "akerror.h"

/* debugging function */
#define akDebugOff(a) printf a
#define akDebug(a)

/* error messages table */
static char *akInternalErrorMessage[] = {
	"internal error",
};
static char *akFatalErrorMessage[] = {
	"fatal error",
	"semaphore",
	"daemon",
	"MPI",
	"mutex lock",
	"mutex unlock",
	"can't find wait context",
	"mutex init",
	"cond init",
	"daemon timer",
	"scheduler",
	"mutex destroy",
	"cond signal",
	"cond destroy",
	"invalid",
	"empty",
	"no memory",
	"used",
	"obsolete",
	"cond wait",
	"get count",
	"context variable",
	"wrong wait context"
};


/*************************************************************************
akErrorPrint - Print the errors in the kernel
*************************************************************************/

extern int akErrorPrint(char *file, int line, int err, char *msg) {
  char c, *m2;

  if (!msg) msg = "no message";

  if (err<=akFATALERROR) 
    fprintf(stderr, "akernel fatal error (%s:%d): %s %s (errno=%d)\n",
    		file, line,
		akFatalErrorMessage[akFATALERROR-err], msg, errno);
  else {
    /* if the error is not fatal, return to caller */
    fprintf(stderr, "akernel error (%s:%d): %s %s (errno=%d)\n", file, line,
		akInternalErrorMessage[akINTERNALERROR-err], msg, errno);
    return err;
  }

  /* blow up everything */
  /*  akFinalize(); */
  exit(err);
}
