/*
 *
 * akInfo.c : Information about thread time usage
 *
 * by Alexandre Carissimi
 * Apache Project, IMAG, Grenoble, France
 * Feb 2, 1999
 *
 */


#include        <sys/lwp.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<unistd.h>
#include	<stdio.h>
#include	<pthread.h>
#include	<procfs.h>
#include	<fcntl.h>
#include	<errno.h>
#include	<string.h>
#include	"akinfo.h" 
    

/**********************
   GLOBAL DEFINITIONS
 **********************/

 
static	TimeStat_t	UsageStat[MAXTHREADS];
static	int		Index = 0;
pthread_mutex_t 	mutex = PTHREAD_MUTEX_INITIALIZER;


#ifdef A0_TSOL2

/*********************************************************************
                          
 *********************************************************************/
 
void	akInfo(int Type)
{
	char		ProcFile[64];
	int		fd, ptr;
	double		tt;
	prusage_t	prusage;
		
	ptr = Type;
	
	switch (Type) {
	case  USER_TYPE:	
		pthread_mutex_lock(&mutex);
		ptr = ptr + Index;
		Index++;
		pthread_mutex_unlock(&mutex);	
		ptr = (ptr >= MAXTHREADS-1)? MAXTHREADS-1 : ptr;
	case  MAIN_TYPE:	
	case  DAEMON_TYPE:
	case  URGENT_TYPE:
	case  SERVICE_TYPE:
	        sprintf(ProcFile,"/proc/%d/lwp/%d/lwpusage",getpid(),_lwp_self());
		break;
	}
	
	if ((fd=open(ProcFile, O_RDONLY)) == -1) {
	   printf("Error %d: %s  on opening %s\n", errno, strerror(errno), ProcFile);
	   return;
	}

	if ( read( fd, (void *)&prusage, sizeof(prusage_t)) == -1) {
	   printf("Error on reading /proc\n");
	   return;
	}
	
       /*
        * Evaluates Elapsed time, user time and system time in microseconds
	*/
	
	UsageStat[ptr].Type = Type;
	
	tt = (double)prusage.pr_rtime.tv_sec*1000000.0+(double)prusage.pr_rtime.tv_nsec/1000.0;
        UsageStat[ptr].ElapsedTime += tt;

	tt = (double)prusage.pr_utime.tv_sec*1000000.0+(double)prusage.pr_utime.tv_nsec/1000.0;
	UsageStat[ptr].UserTime += tt;
	
	tt  = (double)prusage.pr_stime.tv_sec*1000000.0+(double)prusage.pr_stime.tv_nsec/1000.0;
        UsageStat[ptr].SystemTime += tt;

	tt  = (double)prusage.pr_ttime.tv_sec*1000000.0+(double)prusage.pr_ttime.tv_nsec/1000.0;
	tt += (double)prusage.pr_tftime.tv_sec*1000000.0+(double)prusage.pr_tftime.tv_nsec/1000.0;
	tt += (double)prusage.pr_dftime.tv_sec*1000000.0+(double)prusage.pr_dftime.tv_nsec/1000.0;
	tt += (double)prusage.pr_kftime.tv_sec*1000000.0+(double)prusage.pr_kftime.tv_nsec/1000.0;
        UsageStat[ptr].OSTime += tt;
	
	tt  = (double)prusage.pr_ltime.tv_sec*1000000.0+(double)prusage.pr_ltime.tv_nsec/1000.0;
        UsageStat[ptr].LockTime += tt;
	
	tt  = (double)prusage.pr_slptime.tv_sec*1000000.0+(double)prusage.pr_slptime.tv_nsec/1000.0;
        UsageStat[ptr].SlpTime += tt;
	
	tt  = (double)prusage.pr_wtime.tv_sec*1000000.0+(double)prusage.pr_wtime.tv_nsec/1000.0;
        UsageStat[ptr].WaitTime += tt;
	
	tt  = (double)prusage.pr_stoptime.tv_sec*1000000.0+(double)prusage.pr_stoptime.tv_nsec/1000.0;
	UsageStat[ptr].StopTime += tt;	
	
	UsageStat[ptr].Vctx = prusage.pr_vctx;
	
	UsageStat[ptr].Ictx = prusage.pr_ictx;
	
	close(fd);
}
	
/*********************************************************************
                          
 *********************************************************************/
 		
	
void	akPrint(int Node, char *TraceFile)
{
	FILE	*fp;
	char	StatFile[64];
	int	i;
	
	sprintf(StatFile, "%s.n%d", TraceFile, Node);
	   
        if ((fp = fopen(StatFile, "r")) == NULL) {
           fp = fopen(StatFile, "w+");
           if ( fp == NULL) {
              printf("Trace file open error...\n");
              exit(0);
           }
           fprintf(fp, "#\n#      Time Statistics\n");
           fprintf(fp, "#\n# Trace file: %s\n#\n#\n", StatFile);
	}
    	else 
             fp = fopen(StatFile, "a+");
	     
	Index = Index?Index:1;
        fprintf(fp, "------------------------Main------------Daemon------");
	fprintf(fp, "-----Service-----------Urgent------------Users-\n");
        fprintf(fp, "Elapsed Time   :"); for( i=0;i<4+Index; i++)
	fprintf(fp, "%15.4f  ", UsageStat[i].ElapsedTime);
	fprintf(fp, "\n");

	fprintf(fp, "User Time      :"); for( i=0;i<4+Index; i++)
	fprintf(fp, "%15.4f  ", UsageStat[i].UserTime);
	fprintf(fp, "\n");

	fprintf(fp, "System Time    :"); for( i=0;i<4+Index; i++)
	fprintf(fp, "%15.4f  ", UsageStat[i].SystemTime);
	fprintf(fp, "\n");

	fprintf(fp, "OS Time        :"); for( i=0;i<4+Index; i++)
	fprintf(fp, "%15.4f  ", UsageStat[i].OSTime);
	fprintf(fp, "\n");

	fprintf(fp, "Lock Time      :"); for( i=0;i<4+Index; i++)
	fprintf(fp, "%15.4f  ", UsageStat[i].LockTime);
	fprintf(fp, "\n");

	fprintf(fp, "Sleep Time     :"); for( i=0;i<4+Index; i++)
	fprintf(fp, "%15.4f  ", UsageStat[i].SlpTime);
	fprintf(fp, "\n");

	fprintf(fp, "Wait Time      :"); for( i=0;i<4+Index; i++)
	fprintf(fp, "%15.4f  ", UsageStat[i].WaitTime);
	fprintf(fp, "\n");

	fprintf(fp, "Stop Time      :"); for( i=0;i<4+Index; i++)
	fprintf(fp, "%15.4f  ", UsageStat[i].StopTime);
	fprintf(fp, "\n");

	fprintf(fp, "Voluntary ctx  :"); for( i=0;i<4+Index; i++)
	fprintf(fp, "%15.4f  ", (double)UsageStat[i].Vctx);
	fprintf(fp, "\n");

	fprintf(fp, "Involutary ctx :"); for( i=0;i<4+Index; i++)
	fprintf(fp, "%15.4f  ", (double)UsageStat[i].Ictx);
	fprintf(fp, "\n");
        fprintf(fp, "--------------------------------------------------");
	fprintf(fp, "-------------------------------------------------\n");
	    
	fclose(fp);
	
	return;
}
#else

void akInfo(int Type) {
   return;
}

void akPrint(int Node, char *TraceFile) {
   return;
}

#endif
