#include "global_clock.h"
#include <stdio.h>
#include <stdlib.h>
#include "../src/basic/a0_perf_floor_sqrt.h"
#include <ath0.h>

FILE * adjust_input_file;
FILE * adjust_output_file;

int window_size = DEFAULT_WINDOW_SIZE;	/* smoothing window size */
int number_of_points = 0;		/* Number of sample points to use */

statistic *statistic_table;		/* Per node table */

char *content;				/* in memory adjust input file content */
char *current_number, *next_number;	/* positions within the content buffer */

void *memory_alloc (int size)
{
char *mem;
mem = malloc (size);
if (mem == NULL) 
   {
   fprintf (stderr,"Fatal error : cannot allocate memory \n");
   }
return (mem);
}

int allocate_table ()
{
int i;
point *win;
int *rnk;

statistic_table =  (statistic *) memory_alloc  (sizeof (statistic) * 
                                 a0NodeCount);
if (statistic_table == NULL)  return (1);
for (i=0;i<a0NodeCount;i++)
   {
   (statistic_table + i) -> SX = 0.0;
   (statistic_table + i) -> SY = 0.0;
   (statistic_table + i) -> SXX = 0.0;
   (statistic_table + i) -> SXY = 0.0;
   (statistic_table + i) -> SYY = 0.0;
   win = (point *) memory_alloc (sizeof(point) * window_size);
   if (win == NULL)  return (1);
   (statistic_table + i) -> win = win;
   rnk = (int *) memory_alloc (sizeof(int) * window_size);
   if (rnk == NULL)  return (1);
   (statistic_table + i) -> rnk = rnk;
   (statistic_table + i) -> slope = 1.0;
   (statistic_table + i) -> offset = 0.0;
   (statistic_table + i) -> stderr_slope2 = 0.0;
   (statistic_table + i) -> stderr_offset2 = 0.0;
   }
return(0);
}

void print_table()
{
int i;
double std_err;
fprintf (adjust_output_file,"# Athapascan trace global_clock statistics\n#\n");
fprintf (adjust_output_file,"# Number of nodes \n");
fprintf (adjust_output_file,"%d\n#\n",a0NodeCount);
fprintf (adjust_output_file,"# Number of samples points used for the statistics\n");
fprintf (adjust_output_file,"@%d\n#\n",number_of_points);
fprintf (adjust_output_file,"#Reference node sampling time 0 \n");
fprintf (adjust_output_file,"@ %f@\n",node0_initial_time);
fprintf (adjust_output_file,"$ %f$\n\n",node0_initial_time);
fprintf (adjust_output_file,"# Delay between points : %d \n#\n",delay);
fprintf (adjust_output_file,"# Per node statistics\n");
/*
fprintf (adjust_output_file,"# $ slope offset # Std slope err (%%) Std offset err (%%)\n");
*/
fprintf (adjust_output_file,"# $ slope offset # \n");
fprintf (adjust_output_file,"# @ SX SY SXX SXY SYY \n");
for (i=0;i<a0NodeCount;i++)
   {
   fprintf (adjust_output_file,"#\n# Node %4d\n",i);
   fprintf (adjust_output_file,"$ %g\t",(statistic_table + i) -> slope);
   fprintf (adjust_output_file,"%g \t",(statistic_table + i) -> offset);
/*
   std_err = sqrt ((statistic_table + i) -> stderr_slope2);
   fprintf (adjust_output_file,"# %g (%g %%)   ", std_err, 100.0 * std_err / 
                                   (statistic_table + i) -> slope);
   std_err = sqrt ((statistic_table + i) -> stderr_offset2);
   if ((statistic_table + i) -> offset > 1.0e-20)
      fprintf (adjust_output_file,"%g (%g %%)   ", std_err, 100.0 * std_err / 
                                     (statistic_table + i) -> offset);
   else
      fprintf (adjust_output_file,"%g (Not available %%)   ", std_err );
*/
   fprintf (adjust_output_file,"\n");
   fprintf (adjust_output_file,"@ %g\t",(statistic_table + i) -> SX);
   fprintf (adjust_output_file,"%g\t",(statistic_table + i) -> SY);
   fprintf (adjust_output_file,"%g\t",(statistic_table + i) -> SXX);
   fprintf (adjust_output_file,"%g\t",(statistic_table + i) -> SXY);
   fprintf (adjust_output_file,"%g \t",(statistic_table + i) -> SYY);
   fprintf (adjust_output_file,"\n");
   }
fprintf (adjust_output_file,"# End of statistics file\n");
}

void read_table (char *content)
{
char  *buffer_ptr=content;
int c; 
int separator;
int comment;
/* Start : not within a comment, skip trealing separator */
buffer_ptr = content;
separator = 1;
comment = 0;

while ((c = getc(adjust_input_file)) != EOF)
  {
  switch (c)
    {
    case '$'  :
    case '#'  : comment = 1; break;     /* begin or continue comment */

    case '\n' : if (comment) {comment = 0; break;} /* end comment */
                /* If not within a comment, treat as any separator */

    case '@'  :
    case ' '  :
    case '\t' :
    case ','  :
    case ';'  :
    case '{'  :
    case '}'  : if (!comment && !separator)
                   {
                   separator = 1;
                   *buffer_ptr ++ = ' '; /* A single space replace any  */
                   }                      /* number of separators   */
                break;

    default   : if (!comment) 
                    {
                    *buffer_ptr++ = c; 
                    separator = 0;
                    }
    }
  }
}

double read_double (char *message,int node,double *value)
{
  *value = strtod (current_number,&next_number);
  if (next_number == current_number)
    {
    fprintf (stderr,"Adjust file content error (%s) node %d \n",message,node);
    return (1);
    }
  current_number= next_number;
  return (0);
}

/* Read the adjust input file and fills the statistic table */

int read_file_adjust ()
{
int file_size;
int node;
int node_number;
int integer_read;
int return_code = 0;

/* Compute file size */
fseek (adjust_input_file,0,2); 			/* Goto end of file */
file_size = ftell(adjust_input_file);
fseek (adjust_input_file,0,0); 			/* Return to begin of the file */
if (file_size == -1)
   {
   fprintf (stderr,"Cannot get the trace file size and infos\n");
   return_code = 1; goto close_file;
   }

/* Allocate the whole file in memory with a single malloc */
content = memory_alloc(file_size);
if (content == NULL) {return_code = 1 ; goto close_file;}

/* Read file in memory allocated buffer and skip comments */
read_table (content);

current_number = content;

/* Check number of nodes */
node_number = strtol  (current_number,&next_number,10);
if (next_number == current_number)
  {
  fprintf (stderr,"Adjust file content error (number of node missing)\n");
  return_code = 1; goto free_buffer;
  }
else
  current_number= next_number;

if (node_number != a0NodeCount)
  {
  fprintf (stderr,"Fatal error on number of nodes : actual : %d  in file : %d\n",
	  node_number,a0NodeCount);
  return_code = 1; goto free_buffer;
  }

/* Read the number of points */
integer_read = strtol  (current_number,&next_number,10);

number_of_points += integer_read;

if (next_number == current_number)
  {
  fprintf (stderr,"Adjust file content error (number of sample points)\n");
  return_code = 1; goto free_buffer;
  }
else
  current_number= next_number;

/* Read initial_time */
node0_initial_time = strtod (current_number,&next_number);
initial_time_is_defined = 1;

if (next_number == current_number)
  {
  fprintf (stderr,"Adjust file content error (time reference)\n");
  return_code = 1; goto free_buffer;
  }
else
  current_number= next_number;

/* fill adjust array */
for (node = 0; node <a0NodeCount;node++)
  {
if (read_double ("SX",node, &((statistic_table + node) -> SX)))
    {return_code = 1; goto free_buffer;}
if (read_double ("SY",node, &((statistic_table + node) -> SY)))
    {return_code = 1; goto free_buffer;}
if (read_double ("SXX",node, &((statistic_table + node) -> SXX)))
    {return_code = 1; goto free_buffer;}
if (read_double ("SXY",node, &((statistic_table + node) -> SXY)))
    {return_code = 1; goto free_buffer;}
if (read_double ("SYY",node, &((statistic_table + node) -> SYY)))
    {return_code = 1; goto free_buffer;}
  }


/* free memory buffer and close input file */
free_buffer:
if (content != NULL) free(content);
close_file:
if (adjust_input_file != NULL) fclose (adjust_input_file);
return (return_code);
}
