#include "global_clock.h"
#include <ath0.h>
#include "../src/basic/a0_perf_floor_sqrt.h"
#include <sys/time.h>
#include "../src/basic/a0_perf_getime.h"

double node0_initial_time;
int initial_time_is_defined = 0;

int delay = DEFAULT_DELAY;
int number_of_exchanges = DEFAULT_NUMBER_OF_SAMPLES;

a0tBuffer receive_buffer, send_buffer;
a0tPort port;

void print_help ()
{
fprintf (stderr,"Usage : ");
fprintf (stderr,"-if=input_file -of=output_file -ws=window_size\n ");
fprintf (stderr,"-nbp=nb_points -delay=delay  (millisec)\n");
}

int analyse_arguments (int argc,char * argv[])
{
int current_arg;
int value;

/* Output to standard output by default */
adjust_output_file = stdout;
/* No input file by default */
adjust_input_file = NULL;
for (current_arg = 1; current_arg < argc; current_arg++)
  {
  if (!strcmp (argv[current_arg],"-h") || !strcmp (argv[current_arg],"-?"))
     {
     print_help ();
     return (1);
     }
  if (!strncmp (argv[current_arg],"-ws=",4))
     {
     value = atoi (argv[current_arg]+4);
     if ((value >= 1) && (value <= 10))
        window_size = value;
     else
       {
       fprintf (stderr ,"Bad window size [1 .. 10] value : %d\n",value);
       return (1);
       }
     }
  else if (!strncmp (argv[current_arg],"-nbp=",5))
     {
     value = atoi (argv[current_arg]+5);
     if ( (value >0) && (value <= 10000))
        number_of_exchanges = value;
     else
       {
       fprintf (stderr ,"Bad number of sample points [1 .. 10000] : %d\n",value);
       return (1);
       }
     }
   else if (!strncmp (argv[current_arg],"-delay=",7))
     {
     value = atoi (argv[current_arg]+7);
     if ( (value >= 0) && (value <= 10000))
        delay = value;
     else
       {
       fprintf (stderr ,"Bad delay [0 .. 10000] : %d\n",value);
       return (1);
       }
     }
   else if (!strncmp (argv[current_arg],"-if=",4))
     {
     adjust_input_file = fopen (argv[current_arg] + 4,"r");
     if (adjust_input_file == NULL)
        {
        fprintf (stderr,"Cannot open input file : %s\n",argv[current_arg] + 4);
        return (1);
        }
     }
   else if (!strncmp (argv[current_arg],"-of=",4))
     {
     adjust_output_file = fopen (argv[current_arg] + 4,"w+");
     if (adjust_output_file == NULL)
        {
        fprintf (stderr,"Cannot open output file : %s\n",argv[current_arg] + 4);
        return (1);
        }
     }
   else
     {
     fprintf (stderr,"Unknown argument : %s\n",argv[current_arg]);
     print_help ();
     return (1);
     }
  }
return (0);
}

point get_point (int node)
{
point p,p2;
a0ClearBuffer (&send_buffer);
a0ClearBuffer (&receive_buffer);
p2.X = a0_perf_get_node_time () ;
p2.X -= node0_initial_time;
a0Pack (&send_buffer,a0_DOUBLE,&p2.X,1);
a0SendBuffer (&port,node,0,NULL,&send_buffer);
a0ReceiveBuffer (&port,node,0,NULL,&receive_buffer);
a0Unpack (&receive_buffer,a0_DOUBLE,&p.Y,1);
p.X = a0_perf_get_node_time () - node0_initial_time;
p2.Y = (p.X - p.Y) - (p.Y - p2.X);
p.Y += (p.X - p2.X) / 2.0;
return (p);
}

void slave_work ()
{
int nbp;
point p;
/* Receive number of points from the master */
a0ClearBuffer (&receive_buffer);
a0ReceiveBuffer (&port,0,0,NULL,&receive_buffer);
a0Unpack (&receive_buffer,a0_INT,&nbp,1);
a0Unpack (&receive_buffer,a0_DOUBLE,&node0_initial_time,1);
while (nbp >0)
  {
  a0ClearBuffer (&receive_buffer);
  a0ClearBuffer (&send_buffer);
  a0ReceiveBuffer (&port,0,0,NULL,&receive_buffer);
  a0Unpack (&receive_buffer,a0_DOUBLE,&p.X,1);
  p.Y = a0_perf_get_node_time () - node0_initial_time;
  a0Pack (&send_buffer,a0_DOUBLE,&p.Y,1);
  a0SendBuffer (&port,0,0,NULL,&send_buffer);
  nbp --;
  }
a0Terminate ();
}

void master_work (int argc,char *argv[])
{
int i;
int node;
point p;
if (analyse_arguments (argc,argv)) goto error;
if (allocate_table ())             goto error;
if (adjust_input_file != NULL) 
   if (read_file_adjust())	   goto error;

/* Update cumulated number of points (input file + nb_points required) */
number_of_points += number_of_exchanges;

/* We need window_size -1  extra points to fill the window */
number_of_exchanges += window_size - 1;

/* Set initial time */
if (!initial_time_is_defined)
    node0_initial_time = a0_perf_get_node_time ();


for (i=0;i< number_of_exchanges; i++)
  {
  for (node = 1 ; node <a0NodeCount ; node++) 
    {
    if (i == 0)
      {
      int nbp = number_of_exchanges + 1;
      /* Send number of points to slaves */
      a0ClearBuffer (&send_buffer);
      a0Pack (&send_buffer,a0_INT,&nbp,1);
      a0Pack (&send_buffer,a0_DOUBLE,&node0_initial_time,1);
      a0SendBuffer (&port,node,0,NULL,&send_buffer);
      /* get a first extra point to be sure slave is ready to answer */ 
     get_point (node);
      }

    p = get_point (node);
    /* Insert_sort used to fill the window, insert_point when filled */
    if (i< window_size)
      insert_sort (window_size,(statistic_table + node) -> win,
                               (statistic_table + node) -> rnk,p,i);
    else
      insert_point (window_size,(statistic_table + node) -> win,
                              (statistic_table + node) -> rnk,p);
  
    if (i >= window_size -1)
      {
      /* window already filled : get a point */
      get_median_point (window_size,(statistic_table + node) -> win,&p);
      (statistic_table + node) -> SX += p.X;
      (statistic_table + node) -> SY += p.Y;
      (statistic_table + node) -> SXY += p.X * p.Y;
      (statistic_table + node) -> SXX += p.X * p.X;
      (statistic_table + node) -> SYY += p.Y * p.Y;
      }
    }   /* end for nodes */
  usleep (1000 * delay);       /* delay : milliseconds, usleep : microseconds */
  }	/* end for exchanges */
global_compute ();
print_table ();
goto end;

error:

/* Send null number of points to slave */
number_of_exchanges = 0;
for (node = 1; node < a0NodeCount; node++)
  {
  a0ClearBuffer (&send_buffer);
  a0Pack (&send_buffer,a0_INT,&number_of_exchanges,1);
  a0Pack (&send_buffer,a0_DOUBLE,&node0_initial_time,1);
  a0SendBuffer (&port,node,0,NULL,&send_buffer);
  }

end:

a0Terminate ();

}

void  main (int argc, char *argv[])
{

a0Init (&argc,&argv);
a0InitCommit ();
a0NewPort (&port,1,0);
a0NewBuffer (&receive_buffer,A0TypeSendBuffer,100);
a0NewBuffer (&send_buffer,A0TypeSendBuffer,100);
if (a0SelfNode)
  slave_work ();
else
  master_work (argc,argv);
}
