
/*
 *
 * a0fmem.h : a0 Functions of the local memory
 *
 * by Marcelo Pasin
 * Apache Project, IMAG, Grenoble, France
 * Feb 16, 1996
 *
 */


/* Eliminates multiple includes of this file */
#ifndef _A0FMEM_H
#define _A0FMEM_H

#include <ath0.h>
#include "a0fuser.h"

/* Max address size is a constant used to compose the read/write
   request message. In an homogeneous environment, it's equal to the
   size of a pointer. In an heterogeneous environment, the value
   should be changed in compilation time, to a size which can
   store all the pointers existing in the system. Why not 8 or 16?
*/
#ifndef A0MaxAddressSize
#define A0MaxAddressSize sizeof(void *)
#endif


/****************types********************/

/* local address union (a pointer), with byte stuffing for
   heterogeneus future usage */
typedef union a0tMemoryAddress {
    unsigned char bytes[A0MaxAddressSize];
    void	 *address;
} a0tMemoryAddress;


/* DMA region pointer */
typedef struct {
  int node;			/* node that hosts the region */
  a0tMemoryAddress format;	/* format of the region */
  a0tMemoryAddress data;	/* address of the region */
  int size;			/* #of elements of region */
#ifdef A0_TRACE
  int trace_id;
#endif
} a0tDMARegion;



/****************variables********************/

/****************functions********************/

extern a0tError _a0NewDMARegion(a0tDMARegion *region,
			a0tFormat *format, void *pointer, int size);


/* startup function */
extern a0tError _a0UseDMA(void);

/* datatype to send/receive regions */
extern a0tDatatype a0_DMA_REGION;

/* datatype to send/receive addresses */
extern a0tDatatype a0_ADDRESS;

/* ports to communicate */
extern a0tPort _a0MemServerPort, _a0MemAskPort, _a0MemReplyPort;

/* dummy synch variable */
extern int _a0MemDummy;


#ifdef A0_TRACE
#define _a0GetDMARegionId1(region, idp) (*(idp) = region->node)
#define _a0GetDMARegionId2(region, idp) (*(idp) = region->trace_id)
#endif


#ifdef A0F_SOURCE
/****************************************************************************
		Included only for compiling the lib
****************************************************************************/


#define A0ReadRequest	1
#define A0WriteRequest	2
#define A0LockRequest	4
#define A0LockReadRequest (A0LockRequest|A0ReadRequest)
#define A0UnlockRequest	8
#define A0UnlockWriteRequest (A0UnlockRequest|A0WriteRequest)
#define A0MemTerminate	16


typedef struct _a0tMemReply {
  int		type;
  a0tDMARegion	region;
  int		region_start;
  int		region_size;
  int		reply_node;
  int		reply_tag;
  int		writer_node;
  int		writer_tag;
} a0tMemRequest;


/* datatypes to mem requests */
extern a0tDatatype a0_SHORT_MEM_REQUEST, a0_MEM_REQUEST, a0_LONG_MEM_REQUEST;

/* datatype to send/receive regions */
extern a0tDatatype a0_DMA_REGION;

/* datatype to send/receive addresses */
extern a0tDatatype a0_ADDRESS;

/* ports to communicate */
extern a0tPort _a0MemAskPort, _a0MemReplyPort;

/* dummy synch variable */
extern int _a0MemDummy;


/* startup function */
extern a0tError _a0UseDMA(void);

/* find a tag to send/receive */
extern a0tError _a0FindMemTag(int *tagp);


#endif /* A0F_SOURCE */

#endif /* _A0FMEM_H */

