/****************************************************************************/
/* File a0_perf_decode.c					            */
/****************************************************************************/

/*
#define WRITE_IDENTIFY_EVENT
*/

#include "../kernel/akwc.h"
#include "a0_perf_decode.h"
#include "a0thread.h"
#include "a0ports.h"
#include "a0_perf_event_macros.h"
#include "a0_perf_event_codes.h"
#include "a0_perf_trace_buffer.h"

#include <stdio.h>
#include <stdlib.h>

#define print_begin(name,value)\
{\
a0_perf_event_def(name,value);\
a0_perf_write_time("Time");\
a0_perf_write_node();\
a0_perf_write_thread();\
}

FILE *fout,*fin;		

int a0_define_events;
int a0_source=0;
int a0_source_line=0;
int a0_source_file=0;
int a0_node = -1;
int a0_nodecount = -1;
int a0_thread = -1;
int first_file = 1;

double max_time, min_time;

char *a0_perf_index;

char reader_release [A0_PERF_RELEASE_SIZE] = A0_PERF_RELEASE;

double a0_initial_time;

/* Standard a0 primitives event definitions */
/* Header definition with or without source identification */
/* Source : eventdef code + 1 */

int nb_files;
int check_nb_files=0;
file_check check_tab [MAX_NODE_NUMBER];

double time_reference = 0.0;
int time_reference_is_defined = 0;


void a0_perf_event_def (char *name, int value)
{
if (a0_define_events) fprintf (fout,"%%EventDef\t%s\t",name);
fprintf (fout,"%d",value + a0_source);
}

void a0_perf_internal_event_def (char *name,int value)
{
if (a0_define_events && ! a0_source) fprintf(fout,"%%EventDef\t%s\t",name);
fprintf (fout,"%d",value);
}

void a0_perf_event_end ()
{
if (a0_define_events)
  {
  if(a0_source) 
    {
    fprintf (fout,"\n%%\tLine\tint");
    fprintf (fout,"\n%%\tFile\thex");
    }
  fprintf (fout,"\n%%EndEventDef");
  }
else
   if (a0_source) 
     {
     fprintf(fout,"\t%d\t%x",a0_source_line,a0_source_file);
     a0_source = 0;
     }
fprintf (fout,"\n");
}

void a0_perf_internal_event_end ()
{
if (a0_define_events)
    fprintf(fout,"\n%%EndEventDef");
fprintf (fout,"\n");
}

int a0_perf_write_int (char *name)
{
unsigned int value;
if (a0_define_events)
  {
  fprintf (fout,"\n%%\t%s\tint",name);
  }
else
  {
  a0_perf_get_integer (value);
  fprintf (fout,"\t%d",value);
  return (value);
  }
}

void a0_perf_write_node ()
{
if (a0_define_events) fprintf (fout,"\n%%\tNode\tint");
else
  fprintf (fout,"\t%d",a0_node);
}


void a0_perf_write_thread ()
{
if (a0_define_events)  fprintf (fout,"\n%%\tThread\tint");
else
  fprintf (fout,"\t%d",a0_thread);
}


int a0_perf_write_hex (char *name)
{
unsigned int value;
if (a0_define_events) fprintf (fout,"\n%%\t%s\thex",name);
else
  {
  a0_perf_get_integer (value);
  fprintf (fout,"\t%lx",value);
  return (value);
  }
}


void a0_perf_write_port ()
{
unsigned int port_id1,port_id2;
if (a0_define_events) fprintf (fout,"\n%%\tPort\tstring");
else
  {
  a0_perf_get_integer (port_id1);
  a0_perf_get_integer (port_id2);
  fprintf(fout,"\t%d.%d",port_id1,port_id2);
  }
}

void a0_perf_write_region (char *name)
{
unsigned int region_id1,region_id2;
if (a0_define_events) fprintf (fout,"\n%%\tDmaRegion%s\tstring",name);
else
  {
  a0_perf_get_integer (region_id1);
  a0_perf_get_integer (region_id2);
  fprintf(fout,"\t%d.%d",region_id1,region_id2);
  }
}

void a0_perf_write_time (char *name)
{
unsigned int sec, usec;
double t;
if (a0_define_events)
  {
  fprintf (fout,"\n%%\t%s\tdouble",name);
  }
else
  {
  a0_perf_get_integer(sec);
  a0_perf_get_integer(usec);
  /* a0_initial_time form the file */
  /* with offset and time_reference already removed */
  t =  (float) sec  + .000001 * (float) usec + a0_initial_time;
  t /= check_tab [a0_node].slope;
  if (t < min_time) min_time = t;
  if (t > max_time) max_time = t;
  fprintf (fout,"\t%6f",t);
  }
}


void a0_perf_write_string (char *name)
{
char string[2000];
if (a0_define_events) fprintf (fout,"\n%%\t%s\tstring",name);
else
  {
  a0_perf_get_string(string);
  fprintf (fout,"\t\"%s\"",string);
  }
}

void Abort ()
{
print_begin("a0Abort",event_a0Abort);
a0_perf_event_end();
}

void BroadcastCondition ()
{
print_begin("a0BroadcastCondition",event_a0BroadcastCondition);
a0_perf_write_hex("Condition");
a0_perf_event_end();
}

void ClearBuffer ()
{
print_begin("a0ClearBuffer",event_a0ClearBuffer);
a0_perf_write_hex("Buffer");
a0_perf_event_end();
}

void DetachSlave ()
{
print_begin("a0DetachSlave",event_a0DetachSlave);
a0_perf_write_int("OtherThread");
a0_perf_event_end();
}

void DisposeBuffer ()
{
print_begin("a0DisposeBuffer",event_a0DisposeBuffer);
a0_perf_write_hex("Buffer");
a0_perf_event_end();
}

void DisposeCondition ()
{
print_begin("a0DisposeCondition",event_a0DisposeCondition);
a0_perf_write_hex("Condition");
a0_perf_event_end();
}

void DisposeMutex ()
{
print_begin("a0DisposeMutex",event_a0DisposeMutex);
a0_perf_write_hex("Mutex");
a0_perf_event_end();
}

void DisposePort ()
{
print_begin("a0DisposePort",event_a0DisposePort);
a0_perf_write_port();
a0_perf_event_end();
}

void DisposeSemaphore ()
{
print_begin("a0DisposeSemaphore",event_a0DisposeSemaphore);
a0_perf_write_hex("Semaphore");
a0_perf_event_end();
}

void ExitThread ()
{
print_begin("a0ExitThread",event_a0ExitThread);
a0_perf_event_end();
}


void GetRequestData ()
{
print_begin("a0GetRequestData",event_a0GetRequestData);
a0_perf_write_hex("Request");
a0_perf_write_int("SourceNode");
a0_perf_write_int("Size");
a0_perf_write_int("SerialNo");
a0_perf_event_end();
}

void GetThreadPriority ()
{
print_begin("a0GetThreadPriority",event_a0GetThreadPriority);
a0_perf_write_int ("OtherThread");
a0_perf_write_int ("Prio");
a0_perf_event_end();
}

void GetThreadScheduling()
{
print_begin("a0GetThreadScheduling",event_a0GetThreadScheduling);
a0_perf_write_int ("OtherThread");
a0_perf_write_int ("Sched");
a0_perf_event_end();
}

void IdentifyFile ()
{
if (!(a0_define_events && a0_source))
 {
  print_begin("a0_perf_IdentifyFile",a0_perf_IdentifyFile);
  a0_perf_write_hex ("File");
  a0_perf_write_string ("FileName");
  a0_perf_event_end();
 }
}

void Init ()
{
if (!(a0_define_events && a0_source))
  {
  print_begin("a0Init",event_a0Init);
  a0_perf_event_end();
  }
}

void InitCommit ()
{
print_begin("a0InitCommit", event_a0InitCommit);
a0_perf_event_end();
}

void IStartRemoteThread ()
{
print_begin("a0IStartRemoteThread",event_a0IStartRemoteThread);
a0_perf_write_int ("DestNode");
a0_perf_write_int ("Service");
a0_perf_write_int ("Sched");
a0_perf_write_int ("Prio");
a0_perf_write_hex("Stack");
a0_perf_write_hex("Buffer");
a0_perf_write_hex("Request");
a0_perf_write_int ("SerialNo");
a0_perf_event_end();
}

void JoinSlave ()
{
print_begin("a0JoinSlave",event_a0JoinSlave);
a0_perf_write_int ("OtherThread");
a0_perf_write_time ("EndTime");
a0_perf_event_end();
}

void LockMutex ()
{
print_begin("a0LockMutex",event_a0LockMutex);
a0_perf_write_hex("Mutex");
a0_perf_write_time("EndTime");
a0_perf_event_end();
}


void NewBuffer ()
{
print_begin("a0NewBuffer",event_a0NewBuffer);
a0_perf_write_hex("Buffer");
a0_perf_write_int("Buftype");
a0_perf_write_int("Size");
a0_perf_event_end();
}

void NewCondition ()
{
print_begin("a0NewCondition",event_a0NewCondition);
a0_perf_write_hex("Condition");
a0_perf_event_end();
}

void NewMutex ()
{
print_begin("a0NewMutex",event_a0NewMutex);
a0_perf_write_hex("Mutex");
a0_perf_event_end();
}

void NewPort ()
{
print_begin("a0NewPort",event_a0NewPort);
a0_perf_write_port ();
a0_perf_write_int("Tags");
a0_perf_write_int("Unique");
a0_perf_event_end();
}

void NewService()
{
print_begin("a0NewService",event_a0NewService);
a0_perf_write_int("Service");
a0_perf_event_end();
}

void NewSemaphore ()
{
print_begin("a0NewSemaphore",event_a0NewSemaphore);
a0_perf_write_hex("Semaphore");
a0_perf_write_int("Initial");
a0_perf_event_end();
}

void NewSlave ()
{
print_begin("a0NewSlave",event_a0NewSlave);
a0_perf_write_int("OtherThread");
a0_perf_write_int("Sched");
a0_perf_write_int("Prio");
a0_perf_write_hex("Stack");
a0_perf_event_end();
}

void IdentifyEvent ()
{
int usec, sec;
if (!(a0_define_events && a0_source))
 {
  print_begin("a0_perf_IdentifyEvent",a0_perf_IdentifyEvent);
  a0_source_line = a0_perf_write_int ("Line");
  a0_source_file = a0_perf_write_hex ("File");
  a0_perf_event_end();
  }
if (!a0_define_events) a0_source =1;
}  

void Pack()
{
print_begin("a0Pack",event_a0Pack);
a0_perf_write_hex("Buffer");
a0_perf_write_int("Count");
a0_perf_event_end();
}

void Probe ()
{
print_begin("a0Probe",event_a0Probe);
a0_perf_write_int ("SourceNode");
a0_perf_write_int ("Tag");
a0_perf_write_port ();
a0_perf_write_hex ("Request");
a0_perf_write_int("SerialNo");
a0_perf_event_end();
}

void IProbe ()
{
print_begin("a0IProbe",event_a0IProbe);
a0_perf_write_int ("SourceNode");
a0_perf_write_int ("Tag");
a0_perf_write_port ();
a0_perf_write_hex ("Request");
a0_perf_write_int("SerialNo");
a0_perf_event_end();
}

void Receive ()
{
print_begin("a0Receive",event_a0Receive);
a0_perf_write_port();
a0_perf_write_int("SourceNode");
a0_perf_write_int("Tag");
a0_perf_write_hex("Request");
a0_perf_write_int("SerialNo");
a0_perf_write_hex("Address");
a0_perf_write_int("Count");
a0_perf_write_time("EndTime");
a0_perf_event_end();
}

void IReceive ()
{
print_begin("a0IReceive",event_a0IReceive);
a0_perf_write_port();
a0_perf_write_int("SourceNode");
a0_perf_write_int("Tag");
a0_perf_write_hex("Request");
a0_perf_write_int("SerialNo");
a0_perf_write_hex("Address");
a0_perf_write_int("Count");
a0_perf_event_end();
}

void IReceiveAny ()
{
print_begin("a0IReceiveAny",event_a0IReceiveAny);
a0_perf_write_port();
a0_perf_write_int("Tag");
a0_perf_write_hex("Request");
a0_perf_write_int("SerialNo");
a0_perf_write_hex("Address");
a0_perf_write_int("Count");
a0_perf_event_end();
}

void ReceiveBuffer ()
{
print_begin("a0ReceiveBuffer",event_a0ReceiveBuffer);
a0_perf_write_port();
a0_perf_write_int("SourceNode");
a0_perf_write_int("Tag");
a0_perf_write_hex("Request");
a0_perf_write_int("SerialNo");
a0_perf_write_hex("Buffer");
a0_perf_write_time("EndTime");
a0_perf_event_end();
}

void IReceiveBuffer ()
{
print_begin("a0IReceiveBuffer",event_a0IReceiveBuffer);
a0_perf_write_port();
a0_perf_write_int("SourceNode");
a0_perf_write_int("Tag");
a0_perf_write_hex("Request");
a0_perf_write_int("SerialNo");
a0_perf_write_hex("Buffer");
a0_perf_event_end();
}

void IReceiveBufferAny ()
{
print_begin("a0IReceiveBufferAny",event_a0IReceiveBufferAny);
a0_perf_write_port();
a0_perf_write_int("Tag");
a0_perf_write_hex("Request");
a0_perf_write_int("SerialNo");
a0_perf_write_hex("Buffer");
a0_perf_event_end();
}

void PSemaphore ()
{
print_begin("a0PSemaphore",event_a0PSemaphore);
a0_perf_write_hex("Semaphore");
a0_perf_write_time("EndTime");
a0_perf_event_end();
}

void StartRemoteThread ()
{
print_begin("a0StartRemoteThread",event_a0StartRemoteThread);
a0_perf_write_int ("DestNode");
a0_perf_write_int ("Service");
a0_perf_write_int ("Sched");
a0_perf_write_int ("Prio");
a0_perf_write_hex("Stack");
a0_perf_write_hex("Buffer");
a0_perf_write_hex("Request");
a0_perf_write_int ("SerialNo");
a0_perf_event_end();
}

void StartRemoteUrgent ()
{
print_begin("a0StartRemoteUrgent",event_a0StartRemoteUrgent);
a0_perf_write_int("DestNode");
a0_perf_write_int("Service");
a0_perf_write_hex("Buffer");
a0_perf_write_hex("Request");
a0_perf_write_int("SerialNo");
a0_perf_event_end();
}

void IStartRemoteUrgent ()
{
print_begin("a0IStartRemoteUrgent",event_a0IStartRemoteUrgent);
a0_perf_write_int("DestNode");
a0_perf_write_int("Service");
a0_perf_write_hex("Buffer");
a0_perf_write_hex("Request");
a0_perf_write_int("SerialNo");
a0_perf_event_end();
}

void Send ()
{
print_begin("a0Send",event_a0Send);
a0_perf_write_port();
a0_perf_write_int("DestNode");
a0_perf_write_int("Tag");
a0_perf_write_hex("Request");
a0_perf_write_int("SerialNo");
a0_perf_write_hex("Address");
a0_perf_write_int("Count");
a0_perf_write_time("EndTime");
a0_perf_event_end();
}

void ISend ()
{
print_begin("a0ISend",event_a0ISend);
a0_perf_write_port();
a0_perf_write_int("DestNode");
a0_perf_write_int("Tag");
a0_perf_write_hex("Request");
a0_perf_write_int("SerialNo");
a0_perf_write_hex("Address");
a0_perf_write_int("Count");
a0_perf_event_end();
}


void SendBuffer ()
{
print_begin("a0SendBuffer",event_a0SendBuffer);
a0_perf_write_port();
a0_perf_write_int("DestNode");
a0_perf_write_int("Tag");
a0_perf_write_hex("Request");
a0_perf_write_int("SerialNo");
a0_perf_write_hex("Buffer");
a0_perf_write_time("EndTime");
a0_perf_event_end();
}

void ISendBuffer ()
{
print_begin("a0ISendBuffer",event_a0ISendBuffer);
a0_perf_write_port();
a0_perf_write_int("DestNode");
a0_perf_write_int("Tag");
a0_perf_write_hex("Request");
a0_perf_write_int("SerialNo");
a0_perf_write_hex("Buffer");
a0_perf_event_end();
}

void SetDefaultPriority ()
{
print_begin("a0SetDefaultPriority",event_a0SetDefaultPriority);
a0_perf_write_int("Prio");
a0_perf_event_end();
}


void SetDefaultStack ()
{
print_begin("a0SetDefaultStack",event_a0SetDefaultStack);
a0_perf_write_int("Size");
a0_perf_event_end();
}

void SetThreadPriority ()
{
print_begin("a0SetThreadPriority",event_a0SetThreadPriority);
a0_perf_write_int("OtherThread");
a0_perf_write_int("Prio");
a0_perf_event_end();
}


void SetThreadScheduling ()
{
print_begin("a0SetThreadScheduling",event_a0SetThreadScheduling);
a0_perf_write_int("OtherThread");
a0_perf_write_int("Sched");
a0_perf_event_end();
}

void SignalCondition ()
{
print_begin("a0SignalCondition",event_a0SignalCondition);
a0_perf_write_hex("Condition");
a0_perf_event_end();
}

void Terminate ()
{
print_begin("a0Terminate",event_a0Terminate);
a0_perf_write_int("Nb_wait");
a0_perf_event_end();
}

void TestRequest ()
{
print_begin("a0TestRequest",event_a0TestRequest);
a0_perf_write_hex("Request");
a0_perf_write_int("Result");
a0_perf_event_end();
}


void TestAnyRequest ()
{
print_begin("a0TestAnyRequest",event_a0TestAnyRequest);
a0_perf_write_hex("Request");
a0_perf_write_int("Which");
a0_perf_event_end();
}

void UnlockMutex ()
{
print_begin("a0UnlockMutex",event_a0UnlockMutex);
a0_perf_write_hex("Mutex");
a0_perf_event_end();
}

void Unpack()
{
print_begin("a0Unpack",event_a0Unpack);
a0_perf_write_hex("Buffer");
a0_perf_write_int("Count");
a0_perf_event_end();
}

void UserEvent ()
{
print_begin("a0UserEvent",event_a0UserEvent);
a0_perf_write_int ("Integer");
a0_perf_write_string ("String");
a0_perf_event_end();
}

void VSemaphore ()
{
print_begin("a0VSemaphore",event_a0VSemaphore);
a0_perf_write_hex("Semaphore");
a0_perf_event_end();
}

void TryPSemaphore ()
{
print_begin("a0TryPSemaphore",event_a0TryPSemaphore);
a0_perf_write_hex("Semaphore");
a0_perf_write_int("Succeed");
a0_perf_event_end();
}

void TimedWaitCondition ()
{
print_begin("a0TimedWaitCondition",event_a0TimedWaitCondition);
a0_perf_write_hex("Condition");
a0_perf_write_hex("Mutex");
a0_perf_write_int("TimeOut");
a0_perf_event_end();
}

void WaitCondition ()
{
print_begin("a0WaitCondition",event_a0WaitCondition);
a0_perf_write_hex("Condition");
a0_perf_write_hex("Mutex");
a0_perf_event_end();
}

void WaitRequest ()
{
print_begin("a0WaitRequest",event_a0WaitRequest);
a0_perf_write_hex("Request");
a0_perf_write_time("EndTime");
a0_perf_event_end();
}

void WaitAnyRequest ()
{
print_begin("a0WaitAnyRequest",event_a0WaitAnyRequest);
a0_perf_write_hex("Request");
a0_perf_write_int("Which");
a0_perf_write_time("EndTime");
a0_perf_event_end();
}

void YieldThread ()
{
print_begin("a0YieldThread",event_a0YieldThread);
a0_perf_event_end();
}

void NewContext ()
{
print_begin("a0NewContext",event_a0NewContext);
a0_perf_write_hex("Context");
a0_perf_event_end();
}

void GetSelfContext ()
{
print_begin("a0GetSelfContext",event_a0GetSelfContext);
a0_perf_write_hex("Context");
a0_perf_write_hex("Data");
a0_perf_event_end();
}

void SetSelfContext ()
{
print_begin("a0SetSelfContext",event_a0SetSelfContext);
a0_perf_write_hex("Context");
a0_perf_write_hex("Data");
a0_perf_event_end();
}

void GetFormatSize ()
{
print_begin("a0FGetFormatSize",event_a0FGetFormatSize);
a0_perf_write_hex("Format");
a0_perf_write_hex("Address");
a0_perf_write_int("Count");
a0_perf_write_int("Size");
a0_perf_event_end();
}


void GetFormatOffset ()
{
print_begin("a0FGetFormatOffset",event_a0FGetFormatOffset);
a0_perf_write_hex("Format");
a0_perf_write_hex("Address");
a0_perf_write_int("Count");
a0_perf_write_int("Offset");
a0_perf_event_end();
}

void UseFormats ()
{
print_begin("a0UseFormats",event_a0UseFormats);
a0_perf_event_end();
}

void DisposeFormat ()
{
print_begin("a0FDisposeFormat",event_a0FDisposeFormat);
a0_perf_write_hex("Format");
a0_perf_event_end();
}

void IFSend ()
{
print_begin("a0IFSend",event_a0IFSend);
a0_perf_write_port();
a0_perf_write_int("DestNode");
a0_perf_write_int("Tag");
a0_perf_write_hex("Request");
a0_perf_write_int("SerialNo");
a0_perf_write_hex("Format");
a0_perf_write_hex("Address");
a0_perf_write_int("Count");
a0_perf_event_end();
}


void FSend ()
{
print_begin("a0FSend",event_a0FSend);
a0_perf_write_port();
a0_perf_write_int("DestNode");
a0_perf_write_int("Tag");
a0_perf_write_hex("Request");
a0_perf_write_int("SerialNo");
a0_perf_write_hex("Format");
a0_perf_write_hex("Address");
a0_perf_write_int("Count");
a0_perf_write_time ("EndTime");
a0_perf_event_end();
}

void IFReceive ()
{
print_begin("a0IFReceive",event_a0IFReceive);
a0_perf_write_port();
a0_perf_write_int("SourceNode");
a0_perf_write_int("Tag");
a0_perf_write_hex("Request");
a0_perf_write_int("SerialNo");
a0_perf_write_hex("Format");
a0_perf_write_hex("Address");
a0_perf_write_int("Count");
a0_perf_event_end();
}

void IFReceiveAny ()
{
print_begin("a0IFReceiveAny",event_a0IFReceiveAny);
a0_perf_write_port();
a0_perf_write_int("Tag");
a0_perf_write_hex("Request");
a0_perf_write_int("SerialNo");
a0_perf_write_hex("Format");
a0_perf_write_hex("Address");
a0_perf_write_int("Count");
a0_perf_event_end();
}

void FReceive ()
{
print_begin("a0FReceive",event_a0FReceive);
a0_perf_write_port();
a0_perf_write_int("SourceNode");
a0_perf_write_int("Tag");
a0_perf_write_hex("Request");
a0_perf_write_int("SerialNo");
a0_perf_write_hex("Format");
a0_perf_write_hex("Address");
a0_perf_write_int("Count");
a0_perf_write_time ("EndTime");
a0_perf_event_end();
}

void FPack ()
{
print_begin("a0FPack",event_a0FPack);
a0_perf_write_hex("Buffer");
a0_perf_write_hex("Format");
a0_perf_write_hex("Adress");
a0_perf_write_int("Count");
a0_perf_event_end();
}


void FUnpack ()
{
print_begin("a0FUnpack",event_a0FUnpack);
a0_perf_write_hex("Buffer");
a0_perf_write_hex("Format");
a0_perf_write_hex("Adress");
a0_perf_write_int("Count");
a0_perf_event_end();
}


void NewMapFormat ()
{
print_begin("a0NewMapFormat",event_a0NewMapFormat);
a0_perf_write_hex("Format");
a0_perf_write_int("Size");
a0_perf_write_int("Count");
a0_perf_write_hex("Fieldp");
a0_perf_event_end();
}

void SetMapField ()
{
print_begin("a0SetMapField",event_a0SetMapField);
a0_perf_write_hex("Format");
a0_perf_write_hex("Fieldp");
a0_perf_write_hex("BaseAddress");
a0_perf_write_hex("Address");
a0_perf_write_int("Count");
a0_perf_event_end();
}

void NewFunctionFormat ()
{
print_begin("a0NewFunctionFormat",event_a0NewFunctionFormat);
a0_perf_write_hex("Format");
a0_perf_event_end();
}

void IFPut ()
{
print_begin("a0IFPut",event_a0IFPut);
a0_perf_write_hex("Pipe");
a0_perf_write_hex("Request");
a0_perf_write_hex("Format");
a0_perf_write_hex("Adress");
a0_perf_write_int("Count");
a0_perf_event_end ();
}

void FPut ()
{
print_begin("a0FPut",event_a0FPut);
a0_perf_write_hex("Pipe");
a0_perf_write_hex("Format");
a0_perf_write_hex("Adress");
a0_perf_write_int("Count");
a0_perf_write_time ("EndTime");
a0_perf_event_end ();
}

void IFGet ()
{
print_begin("a0IFGet",event_a0IFGet);
a0_perf_write_hex("Pipe");
a0_perf_write_hex("Request");
a0_perf_write_hex("Format");
a0_perf_write_hex("Adress");
a0_perf_write_int("Count");
a0_perf_event_end ();
}

void FGet ()
{
print_begin("a0FGet",event_a0FGet);
a0_perf_write_hex("Pipe");
a0_perf_write_hex("Format");
a0_perf_write_hex("Adress");
a0_perf_write_int("Count");
a0_perf_write_time ("EndTime");
a0_perf_event_end ();
}

void Flush ()
{
print_begin("a0Flush",event_a0Flush);
a0_perf_write_hex("Pipe");
a0_perf_write_time ("EndTime");
a0_perf_event_end ();
}

void NewPipe ()
{
print_begin("a0NewPipe",event_a0NewPipe);
a0_perf_write_hex("Pipe");
a0_perf_write_port();
a0_perf_write_int ("OtherNode");
a0_perf_write_int ("Tag");
a0_perf_write_int ("BufSize");
a0_perf_write_int ("Protocol");
a0_perf_write_int ("Prio");
a0_perf_write_int ("OtherThread");
a0_perf_write_hex ("Buffer");
a0_perf_event_end ();
}

void DisposePipe ()
{
print_begin("a0DisposePipe",event_a0DisposePipe);
a0_perf_write_hex("Pipe");
a0_perf_event_end ();
}

void NewDmaRegion ()
{
print_begin("a0NewDmaRegion",event_a0NewDmaRegion);
a0_perf_write_region ("");
a0_perf_write_hex("Format");
a0_perf_write_hex("Address");
a0_perf_write_int ("Size");
a0_perf_event_end ();
}

void IFReadPart ()
{
print_begin("a0IFReadPart",event_a0IFReadPart);
a0_perf_write_hex("Format");
a0_perf_write_region ("");
a0_perf_write_hex("StartAddress");
a0_perf_write_int ("Size");
a0_perf_write_hex("Request");
a0_perf_write_int("SerialNo");
a0_perf_write_hex("Address");
a0_perf_write_int ("Count");
a0_perf_event_end ();
}

void FReadPart ()
{
print_begin("a0FReadPart",event_a0FReadPart);
a0_perf_write_hex("Format");
a0_perf_write_region ("");
a0_perf_write_hex("StartAddress");
a0_perf_write_int ("Size");
a0_perf_write_hex("Request");
a0_perf_write_int("SerialNo");
a0_perf_write_hex("Address");
a0_perf_write_int ("Count");
a0_perf_write_time ("EndTime");
a0_perf_event_end ();
}


void IFRead ()
{
print_begin("a0IFRead",event_a0IFRead);
a0_perf_write_hex("Format");
a0_perf_write_region ("");
a0_perf_write_hex("Request");
a0_perf_write_int("SerialNo");
a0_perf_write_hex("Address");
a0_perf_write_int ("Count");
a0_perf_event_end ();
}

void FRead ()
{
print_begin("a0FRead",event_a0FRead);
a0_perf_write_hex("Format");
a0_perf_write_region ("");
a0_perf_write_hex("Request");
a0_perf_write_int("SerialNo");
a0_perf_write_hex("Address");
a0_perf_write_int ("Count");
a0_perf_write_time ("EndTime");
a0_perf_event_end ();
}

void IFWritePart ()
{
print_begin("a0IFWritePart",event_a0IFWritePart);
a0_perf_write_hex("Format");
a0_perf_write_region ("");
a0_perf_write_hex("StartAddress");
a0_perf_write_int ("Size");
a0_perf_write_hex("Request");
a0_perf_write_int("SerialNo");
a0_perf_write_hex("Address");
a0_perf_write_int ("Count");
a0_perf_event_end ();
}

void FWritePart ()
{
print_begin("a0FWritePart",event_a0FWritePart);
a0_perf_write_hex("Format");
a0_perf_write_region ("");
a0_perf_write_hex("StartAddress");
a0_perf_write_int ("Size");
a0_perf_write_hex("Request");
a0_perf_write_int("SerialNo");
a0_perf_write_hex("Address");
a0_perf_write_int ("Count");
a0_perf_write_time ("EndTime");
a0_perf_event_end ();
}


void IFWrite ()
{
print_begin("a0IFWrite",event_a0IFWrite);
a0_perf_write_hex("Format");
a0_perf_write_region ("");
a0_perf_write_hex("Request");
a0_perf_write_int("SerialNo");
a0_perf_write_hex("Address");
a0_perf_write_int ("Count");
a0_perf_event_end ();
}

void FWrite ()
{
print_begin("a0FWrite",event_a0FWrite);
a0_perf_write_hex("Format");
a0_perf_write_region ("");
a0_perf_write_hex("Request");
a0_perf_write_int("SerialNo");
a0_perf_write_hex("Address");
a0_perf_write_int ("Count");
a0_perf_write_time ("EndTime");
a0_perf_event_end ();
}

void IFCopyPart ()
{
print_begin("a0IFCopyPart",event_a0IFCopyPart);
a0_perf_write_region ("1");
a0_perf_write_hex("StartAddress1");
a0_perf_write_int ("Size1");
a0_perf_write_hex("Request");
a0_perf_write_int("SerialNo");
a0_perf_write_region ("2");
a0_perf_write_hex("StartAddress2");
a0_perf_write_int ("Size2");
a0_perf_event_end ();
}

void FCopyPart ()
{
print_begin("a0FCopyPart",event_a0FCopyPart);
a0_perf_write_region ("1");
a0_perf_write_hex("StartAddress1");
a0_perf_write_int ("Size1");
a0_perf_write_hex("Request");
a0_perf_write_int("SerialNo");
a0_perf_write_region ("2");
a0_perf_write_hex("StartAddress2");
a0_perf_write_int ("Size2");
a0_perf_write_time ("EndTime");
a0_perf_event_end ();
}

void IFCopy ()
{
print_begin("a0IFCopy",event_a0IFCopy);
a0_perf_write_region ("1");
a0_perf_write_hex("Request");
a0_perf_write_int("SerialNo");
a0_perf_write_region ("2");
a0_perf_event_end ();
}

void FCopy ()
{
print_begin("a0FCopy",event_a0FCopy);
a0_perf_write_region ("1");
a0_perf_write_hex("Request");
a0_perf_write_int("SerialNo");
a0_perf_write_region ("2");
a0_perf_write_time ("EndTime");
a0_perf_event_end ();
}

void SetLockRead ()
{
print_begin("a0SetLockRead",event_a0SetLockRead);
/* 
a0_perf_write_hex("Lock");
a0_perf_write_int("Index");
a0_perf_write_hex("Formatp");
a0_perf_write_hex("Address");
a0_perf_write_int("Count");
*/
a0_perf_event_end ();
}

void SetUnlockWrite ()
{
print_begin("a0SetUnlockWrite",event_a0SetUnlockWrite);
/*
a0_perf_write_hex("Lock");
a0_perf_write_int("Index");
a0_perf_write_hex("Formatp");
a0_perf_write_hex("Address");
a0_perf_write_int("Count");
*/
a0_perf_event_end ();
}

void SetLockDMARegion ()
{
print_begin("a0SetLockDMARegion",event_a0SetLockDMARegion);
/*
a0_perf_write_hex("Lock");
a0_perf_write_int("Index");
a0_perf_write_region("");
*/
a0_perf_event_end ();
}

void SetLockDMARegionPart ()
{
print_begin("a0SetLockDMARegionPart",event_a0SetLockDMARegionPart);
/*
a0_perf_write_hex("Lock");
a0_perf_write_int("Index");
a0_perf_write_region("");
a0_perf_write_int("First");
a0_perf_write_int("Size");
*/
a0_perf_event_end ();
}

void Lock ()
{
print_begin("a0Lock",event_a0Lock);
/*
a0_perf_write_hex("Lock");
a0_perf_write_int("Count");
a0_perf_write_hex("Requestp");
*/
a0_perf_write_time ("EndTime");
a0_perf_event_end ();
}

void ILock ()
{
print_begin("a0ILock",event_a0ILock);
/*
a0_perf_write_hex("Lock");
a0_perf_write_int("Count");
a0_perf_write_hex("Requestp");
*/
a0_perf_event_end ();
}

void Unlock ()
{
print_begin("a0Unlock",event_a0Unlock);
/*
a0_perf_write_hex("Lock");
a0_perf_write_int("Count");
a0_perf_write_hex("Requestp");
*/
a0_perf_write_time ("EndTime");
a0_perf_event_end ();
}

void IUnlock ()
{
print_begin("a0IUnlock",event_a0IUnlock);
/*
a0_perf_write_hex("Lock");
a0_perf_write_int("Count");
a0_perf_write_hex("Requestp");
*/
a0_perf_event_end ();
}

void IFReadPartLock ()
{
print_begin("a0IFReadPartLock",event_a0IFReadPartLock);
/*
a0_perf_write_hex("Lock");
a0_perf_write_region("");
a0_perf_write_int("Start");
a0_perf_write_int("Size");
a0_perf_write_hex("Reqp");
a0_perf_write_hex("Formatp");
a0_perf_write_hex("Address");
a0_perf_write_int("Count");
*/
a0_perf_event_end ();
}

void FReadPartLock ()
{
print_begin("a0FReadPartLock",event_a0FReadPartLock);
/*
a0_perf_write_hex("Lock");
a0_perf_write_region("");
a0_perf_write_int("Start");
a0_perf_write_int("Size");
a0_perf_write_hex("Reqp");
a0_perf_write_hex("Formatp");
a0_perf_write_hex("Address");
a0_perf_write_int("Count");
*/
a0_perf_write_time ("EndTime");
a0_perf_event_end ();
}

void FWriteUnlock ()
{
print_begin("a0FWriteUnlock",event_a0FWriteUnlock);
/*
a0_perf_write_hex("Lock");
a0_perf_write_hex("Reqp");
a0_perf_write_hex("Formatp");
a0_perf_write_hex("Address");
a0_perf_write_int("Count");
*/
a0_perf_write_time ("EndTime");
a0_perf_event_end ();
}

void IFWriteUnlock ()
{
print_begin("a0IFWriteUnlock",event_a0IFWriteUnlock);
/*
a0_perf_write_hex("Lock");
a0_perf_write_hex("Reqp");
a0_perf_write_hex("Formatp");
a0_perf_write_hex("Address");
a0_perf_write_int("Count");
*/
a0_perf_event_end ();
}

void BeginThreadService ()
{
if (!(a0_define_events && a0_source))
  {
  /* No source information for internal events */
  a0_perf_internal_event_def ("a0_perf_BeginThreadService",
                               a0_perf_BeginThreadService);
  a0_perf_write_time ("Time");
  a0_perf_write_node();
  a0_perf_write_thread ();
  a0_perf_write_int("Service");
  a0_perf_write_int("StartedThread");
  a0_perf_write_int("SourceNode");
  a0_perf_write_int("SerialNo");
  /* Reset source mode */
  a0_source = 0;
  a0_perf_internal_event_end();
  }
}

void BeginUrgentService ()
{
if (!(a0_define_events && a0_source))
  {
  /* No source information for internal events */
  a0_perf_internal_event_def ("a0_perf_BeginUrgentService",
                               a0_perf_BeginUrgentService);
  a0_perf_write_time ("Time");
  a0_perf_write_node();
  a0_perf_write_thread();
  a0_perf_write_int("Service");
  a0_perf_write_int("SourceNode");
  a0_perf_write_int("SerialNo");
  /* Reset source mode */
  a0_source = 0;
  a0_perf_internal_event_end();
  }
}

void EndUrgentService ()
{
if (!(a0_define_events && a0_source))
  {
  /* No source information for internal events */
  a0_perf_internal_event_def ("a0_perf_EndUrgentService",
                               a0_perf_EndUrgentService);
  a0_perf_write_time ("Time");
  a0_perf_write_node();
  a0_perf_write_thread ();
  a0_perf_internal_event_end();
  }
}


void EndOfUserThread ()
{
if (!(a0_define_events && a0_source))
  {
  /* No source information for internal events */
  a0_perf_internal_event_def ("a0_perf_EndOfUserThread",
                              a0_perf_EndOfUserThread);
  a0_perf_write_time ("Time");
  a0_perf_write_node();
  a0_perf_write_thread();
  a0_perf_internal_event_end();
  }
}

void WaitTraceBuffer ()
{
if (!(a0_define_events && a0_source))
  {
  /* No source information for internal events */
  a0_perf_internal_event_def ("a0_perf_WaitTraceBuffer",
		               a0_perf_WaitTraceBuffer);
  a0_perf_write_time ("Time");
  a0_perf_write_node();
  a0_perf_write_thread();
  a0_perf_internal_event_end();
  }
}

void InitUserThreadTraceBuffer ()
{
if (!(a0_define_events && a0_source)) 
  /* No source information for internal events */
  {
  a0_perf_internal_event_def ("a0_perf_InitUserThreadTraceBuffer",
  				a0_perf_InitUserThreadTraceBuffer);
  a0_perf_write_time ("Time");
  a0_perf_write_node();
  /* Set the thread number to which the buffer belongs */
  /* Only when reading the trace file, not when writing header*/
  if (!a0_define_events) a0_perf_get_integer(a0_thread);
  /* Updates current thread */
  a0_perf_write_thread();
  /* Reset source info mode, only when decoding content also */
  if (!a0_define_events) a0_source = 0;
  a0_perf_internal_event_end();
  }
}

void CompletedRequest (int code)
{
int is_recv = 0;
if (!(a0_define_events && a0_source))
  /* No source information for internal events */
  {
  switch (code)
    {
    case A0_PERF_RECORD_ID_COMPLETED_RECV:
        a0_perf_internal_event_def ("a0_perf_CompletedRecv",a0_perf_CompletedRecv);
	is_recv = 1;
	break;
    case A0_PERF_RECORD_ID_COMPLETED_SEND:
        a0_perf_internal_event_def ("a0_perf_CompletedSend",a0_perf_CompletedSend);
	break;
    case A0_PERF_RECORD_ID_COMPLETED_PROBE:
        a0_perf_internal_event_def ("a0_perf_CompletedProbe",a0_perf_CompletedProbe);
	break;
    case A0_PERF_RECORD_ID_COMPLETED_OTHER:
        a0_perf_internal_event_def ("a0_perf_CompletedOther",a0_perf_CompletedOther);
	break;
    }
  a0_perf_write_time ("Time");
  a0_perf_write_node();
  a0_perf_write_thread();
  a0_perf_write_int("SerialNo");
  a0_perf_write_int("Size");
  if (is_recv) a0_perf_write_int ("SourceNode");
  a0_perf_internal_event_end();
  }
}

void IdentifyFormat ()
{
if (!(a0_define_events && a0_source))
  {
  a0_perf_internal_event_def ("a0_perf_IdentifyFormat",a0_perf_IdentifyFormat);
  a0_perf_write_time ("Time");
  a0_perf_write_node();
  a0_perf_write_thread();
  a0_perf_write_hex ("Format");
  a0_perf_write_string ("FormatName");
  a0_perf_internal_event_end();
  }
}

void InitFRead ()
{
if (!(a0_define_events && a0_source))
  {
  a0_perf_internal_event_def ("a0_perf_InitFRead",a0_perf_InitFRead);
  a0_perf_write_time ("Time");
  a0_perf_write_node();
  a0_perf_write_thread();
  a0_perf_write_int ("SourceNode");
  a0_perf_write_int ("Tag");
  a0_perf_write_int ("RTCSerialNo");
  a0_perf_internal_event_end();
  }
}

void InitFWrite ()
{
if (!(a0_define_events && a0_source))
  {
  a0_perf_internal_event_def ("a0_perf_InitFWrite",a0_perf_InitFWrite);
  a0_perf_write_time ("Time");
  a0_perf_write_node();
  a0_perf_write_thread();
  a0_perf_write_int ("DestNode");
  a0_perf_write_int ("Tag");
  a0_perf_write_int ("RTCSerialNo");
  a0_perf_write_int ("SerialNo");
  a0_perf_internal_event_end();
  }
}

void InitFCopy ()
{
if (!(a0_define_events && a0_source))
  {
  a0_perf_internal_event_def ("a0_perf_InitFCopy",a0_perf_InitFCopy);
  a0_perf_write_time ("Time");
  a0_perf_write_node();
  a0_perf_write_thread();
  a0_perf_write_int ("SourceNode");
  a0_perf_write_int ("DestNode");
  a0_perf_write_int ("Tag");
  a0_perf_write_int ("RTCSerialNo");
  a0_perf_internal_event_end();
  }
}

void InitFLock ()
{
if (!(a0_define_events && a0_source))
  {
  a0_perf_internal_event_def ("a0_perf_InitFLock",a0_perf_InitFLock);
  a0_perf_write_time ("Time");
  a0_perf_write_node();
  a0_perf_write_thread();
  a0_perf_write_int ("OtherNode");
  a0_perf_write_int ("Tag");
  a0_perf_write_int ("RTCSerialNo");
  a0_perf_internal_event_end();
  }
}

void InitFLockRead ()
{
if (!(a0_define_events && a0_source))
  {
  a0_perf_internal_event_def ("a0_perf_InitFLockread",a0_perf_InitFLockRead);
  a0_perf_write_time ("Time");
  a0_perf_write_node();
  a0_perf_write_thread();
  a0_perf_write_int ("OtherNode");
  a0_perf_write_int ("Tag");
  a0_perf_write_int ("RTCSerialNo");
  a0_perf_write_int ("SerialNo");
  a0_perf_internal_event_end();
  }
}

void InitFUnlock ()
{
if (!(a0_define_events && a0_source))
  {
  a0_perf_internal_event_def ("a0_perf_InitFUnlock",a0_perf_InitFUnlock);
  a0_perf_write_time ("Time");
  a0_perf_write_node();
  a0_perf_write_thread();
  a0_perf_write_int ("OtherNode");
  a0_perf_write_int ("Tag");
  a0_perf_write_int ("RTCSerialNo");
  a0_perf_internal_event_end();
  }
}

void InitFUnlockWrite ()
{
if (!(a0_define_events && a0_source))
  {
  a0_perf_internal_event_def ("a0_perf_InitFUnlockWrite",
				a0_perf_InitFUnlockWrite);
  a0_perf_write_time ("Time");
  a0_perf_write_node();
  a0_perf_write_thread();
  a0_perf_write_int ("OtherNode");
  a0_perf_write_int ("Tag");
  a0_perf_write_int ("RTCSerialNo");
  a0_perf_write_int ("SerialNo");
  a0_perf_internal_event_end();
  }
}

void WriteCopy ()
{
if (!(a0_define_events && a0_source))
  {
  a0_perf_internal_event_def ("a0_perf_WriteCopy",
				a0_perf_WriteCopy);
  a0_perf_write_time ("Time");
  a0_perf_write_node();
  a0_perf_write_thread();
  a0_perf_write_int ("OtherNode");
  a0_perf_write_int ("Tag");
  a0_perf_write_int ("RTCSerialNo");
  a0_perf_write_int ("SerialNo");
  a0_perf_internal_event_end();
  }
}

void ReplyRead ()
{
if (!(a0_define_events && a0_source))
  {
  a0_perf_internal_event_def ("a0_perf_ReplyRead",
				a0_perf_ReplyRead);
  a0_perf_write_time ("Time");
  a0_perf_write_node();
  a0_perf_write_thread();
  a0_perf_write_int ("OtherNode");
  a0_perf_write_int ("Tag");
  a0_perf_write_int ("SerialNo");
  a0_perf_internal_event_end();
  }
}

void ReplyWrite ()
{
if (!(a0_define_events && a0_source))
  {
  a0_perf_internal_event_def ("a0_perf_ReplyWrite",
				a0_perf_ReplyWrite);
  a0_perf_write_time ("Time");
  a0_perf_write_node();
  a0_perf_write_thread();
  a0_perf_write_int ("OtherNode");
  a0_perf_write_int ("Tag");
  a0_perf_write_int ("RecvSerialNo");
  a0_perf_write_int ("SerialNo");
  a0_perf_internal_event_end();
  }
}
void SetLimits()
{
if (a0_define_events && !a0_source)
  {
  /* Pseudo event with min time max_time and NodeNumber */
  fprintf (fout,"%%EventDef\tSetLimits\t0");
  fprintf (fout,"\n%%\tTime\tdouble");
  fprintf (fout,"\n%%\tNode\tint");
  fprintf (fout,"\n%%\tThread\tint");
  fprintf (fout,"\n%%\tStartTime\tdouble");
  fprintf (fout,"\n%%\tEndTime\tdouble");
  fprintf (fout,"\n%%\tNodeNumber\tint");
  a0_perf_internal_event_end();
  }
if (!a0_define_events)
  fprintf (fout,"%d\t%6f\t0\t0\t%6f\t%6f\t%d\n",
           event_a0SetLimits,min_time - 0.000001,min_time,max_time,a0_nodecount);
}


int a0_perf_decode_event (char **buffer)
{
unsigned char event;
a0_perf_index = *buffer;
a0_perf_get_event_id (a0_perf_index,event);
switch (event)
  {
  case A0_PERF_RECORD_ID_ABORT: Abort (); break;
  case A0_PERF_RECORD_ID_BROADCASTCONDITION: BroadcastCondition (); break;
  case A0_PERF_RECORD_ID_DISPOSEPORT: DisposePort (); break;
  case A0_PERF_RECORD_ID_DETACHSLAVE: DetachSlave (); break;
  case A0_PERF_RECORD_ID_DISPOSEBUFFER: DisposeBuffer (); break;
  case A0_PERF_RECORD_ID_DISPOSECONDITION: DisposeCondition (); break;
  case A0_PERF_RECORD_ID_DISPOSEMUTEX: DisposeMutex (); break;
  case A0_PERF_RECORD_ID_DISPOSESEMAPHORE: DisposeSemaphore (); break;
  case A0_PERF_RECORD_ID_EXITTHREAD: ExitThread (); break;
  case A0_PERF_RECORD_ID_GETREQUESTINFO: GetRequestData (); break;
  case A0_PERF_RECORD_ID_GETTHREADPRIORITY: GetThreadPriority (); break;
  case A0_PERF_RECORD_ID_GETTHREADSCHEDULING: GetThreadScheduling (); break;
  case A0_PERF_RECORD_ID_INIT: Init (); break;
  case A0_PERF_RECORD_ID_INITCOMMIT: InitCommit (); break;
  case A0_PERF_RECORD_ID_JOINSLAVE: JoinSlave (); break;
  case A0_PERF_RECORD_ID_LOCKMUTEX: LockMutex (); break;
  case A0_PERF_RECORD_ID_NEWBUFFER: NewBuffer (); break;
  case A0_PERF_RECORD_ID_NEWCONDITION: NewCondition (); break;
  case A0_PERF_RECORD_ID_NEWMUTEX: NewMutex (); break;
  case A0_PERF_RECORD_ID_NEWPORT: NewPort (); break;
  case A0_PERF_RECORD_ID_NEWSEMAPHORE: NewSemaphore (); break;
  case A0_PERF_RECORD_ID_NEWRRSERVICE: NewService (); break;
  case A0_PERF_RECORD_ID_NEWSLAVE: NewSlave (); break;
  case A0_PERF_RECORD_ID_PACK: Pack (); break;
  case A0_PERF_RECORD_ID_PSEMAPHORE: PSemaphore (); break;
  case A0_PERF_RECORD_ID_RECEIVE: Receive (); break;
  case A0_PERF_RECORD_ID_RECEIVEBUFFER: ReceiveBuffer (); break;
  case A0_PERF_RECORD_ID_IRECEIVE: IReceive (); break;
  case A0_PERF_RECORD_ID_IRECEIVEANY: IReceiveAny (); break;
  case A0_PERF_RECORD_ID_IRECEIVEBUFFER: IReceiveBuffer (); break;
  case A0_PERF_RECORD_ID_IRECEIVEBUFFERANY: IReceiveBufferAny (); break;
  case A0_PERF_RECORD_ID_REMOTEREQUEST: StartRemoteUrgent (); break;
  case A0_PERF_RECORD_ID_REMOTETHREADCREATE: StartRemoteThread(); break;
  case A0_PERF_RECORD_ID_IREMOTEREQUEST: IStartRemoteUrgent (); break;
  case A0_PERF_RECORD_ID_IREMOTETHREADCREATE: IStartRemoteThread(); break;
  case A0_PERF_RECORD_ID_SEND: Send (); break;
  case A0_PERF_RECORD_ID_ISEND: ISend (); break;
  case A0_PERF_RECORD_ID_SENDBUFFER: SendBuffer (); break;
  case A0_PERF_RECORD_ID_ISENDBUFFER: ISendBuffer (); break;
  case A0_PERF_RECORD_ID_SETDEFAULTPRIORITY: SetDefaultPriority (); break;
/*
  case A0_PERF_RECORD_ID_SETDEFAULTSCHEDULING: SetDefaultScheduling (); break;
*/
  case A0_PERF_RECORD_ID_SETDEFAULTSTACK: SetDefaultStack (); break;
  case A0_PERF_RECORD_ID_SETTHREADPRIORITY: SetThreadPriority (); break;
  case A0_PERF_RECORD_ID_SETTHREADSCHEDULING: SetThreadScheduling (); break;
  case A0_PERF_RECORD_ID_SIGNALCONDITION: SignalCondition (); break;
  case A0_PERF_RECORD_ID_TERMINATE: Terminate (); break;
  case A0_PERF_RECORD_ID_TESTREQUEST: TestRequest (); break;
  case A0_PERF_RECORD_ID_TIMEDWAITCONDITION: TimedWaitCondition (); break;
  case A0_PERF_RECORD_ID_TRYPSEMAPHORE: TryPSemaphore (); break;
  case A0_PERF_RECORD_ID_UNLOCKMUTEX: UnlockMutex (); break;
  case A0_PERF_RECORD_ID_UNPACK: Unpack (); break;
  case A0_PERF_RECORD_ID_VSEMAPHORE: VSemaphore (); break;
  case A0_PERF_RECORD_ID_WAITCONDITION: WaitCondition (); break;
  case A0_PERF_RECORD_ID_WAITREQUEST: WaitRequest (); break;
  case A0_PERF_RECORD_ID_YIELDTHREAD: YieldThread (); break;
  case A0_PERF_RECORD_ID_NEWCONTEXT: NewContext (); break;
  case A0_PERF_RECORD_ID_GETSELFCONTEXT: GetSelfContext (); break;
  case A0_PERF_RECORD_ID_SETSELFCONTEXT: SetSelfContext (); break;

  case A0_PERF_RECORD_ID_INIT_TRACE_BUFFER: InitUserThreadTraceBuffer (); break;
  case A0_PERF_RECORD_ID_END_OF_USER_THREAD: EndOfUserThread (); break;
  case A0_PERF_RECORD_ID_USER_EVENT: UserEvent (); break;
  case A0_PERF_RECORD_ID_START_REMOTE_SERVICE: BeginThreadService (); break;
  case A0_PERF_RECORD_ID_BEGIN_REMOTE_REQUEST: BeginUrgentService (); break;
  case A0_PERF_RECORD_ID_END_REMOTE_REQUEST: EndUrgentService (); break;
  case A0_PERF_RECORD_ID_WAIT_BUFFER: WaitTraceBuffer (); break;
  case A0_PERF_RECORD_ID_IDENTIFY_EVENT: IdentifyEvent(); break;
  case A0_PERF_RECORD_ID_IDENTIFY_FILE: IdentifyFile (); break;
  case A0_PERF_RECORD_ID_IDENTIFY_FORMAT: IdentifyFormat (); break;
  case A0_PERF_RECORD_ID_CLEAR_BUFFER: ClearBuffer (); break;
  case A0_PERF_RECORD_ID_PROBE: Probe ();break;
  case A0_PERF_RECORD_ID_IPROBE: IProbe ();break;
  case A0_PERF_RECORD_ID_COMPLETED_SEND:
  case A0_PERF_RECORD_ID_COMPLETED_RECV:
  case A0_PERF_RECORD_ID_COMPLETED_PROBE:
  case A0_PERF_RECORD_ID_COMPLETED_OTHER: 
			CompletedRequest (event); break;
  case A0_PERF_RECORD_ID_GETFORMATSIZE: GetFormatSize(); break;
  case A0_PERF_RECORD_ID_GETFORMATOFFSET: GetFormatOffset(); break;
  case A0_PERF_RECORD_ID_DISPOSEFORMAT: DisposeFormat(); break;
  case A0_PERF_RECORD_ID_NEWMAPFORMAT: NewMapFormat(); break;
  case A0_PERF_RECORD_ID_SETMAPFIELD: SetMapField(); break;
  case A0_PERF_RECORD_ID_NEWFUNCTIONFORMAT: NewFunctionFormat (); break;
  case A0_PERF_RECORD_ID_WAITANYREQUEST: WaitAnyRequest (); break;
  case A0_PERF_RECORD_ID_TESTANYREQUEST: TestAnyRequest (); break;
  case A0_PERF_RECORD_ID_IFPUT: IFPut ();break; break;
  case A0_PERF_RECORD_ID_FPUT: FPut ();break; break;
  case A0_PERF_RECORD_ID_IFGET: IFGet (); break; break;
  case A0_PERF_RECORD_ID_FGET: FGet (); break; break;
  case A0_PERF_RECORD_ID_FLUSH: Flush ();break; break;
  case A0_PERF_RECORD_ID_NEWPIPE: NewPipe (); break; break;
  case A0_PERF_RECORD_ID_DISPOSEPIPE: DisposePipe (); break;
  case A0_PERF_RECORD_ID_USEFORMATS: UseFormats (); break;
  case A0_PERF_RECORD_ID_IFSEND: IFSend (); break;
  case A0_PERF_RECORD_ID_FSEND: FSend (); break;
  case A0_PERF_RECORD_ID_IFRECEIVE: IFReceive (); break;
  case A0_PERF_RECORD_ID_FRECEIVE: FReceive (); break;
  case A0_PERF_RECORD_ID_IFRECEIVEANY: IFReceiveAny (); break;
  case A0_PERF_RECORD_ID_FPACK:FPack (); break;
  case A0_PERF_RECORD_ID_FUNPACK: FUnpack (); break;
  case A0_PERF_RECORD_ID_NEWDMAREGION: NewDmaRegion (); break;
  case A0_PERF_RECORD_ID_IFREADPART: IFReadPart (); break;
  case A0_PERF_RECORD_ID_FREADPART: FReadPart (); break;
  case A0_PERF_RECORD_ID_IFREAD: IFRead (); break;
  case A0_PERF_RECORD_ID_FREAD: FRead (); break;
  case A0_PERF_RECORD_ID_IFWRITEPART: IFWritePart (); break;
  case A0_PERF_RECORD_ID_FWRITEPART: FWritePart (); break;
  case A0_PERF_RECORD_ID_IFWRITE: IFWrite (); break;
  case A0_PERF_RECORD_ID_FWRITE: FWrite (); break;
  case A0_PERF_RECORD_ID_IFCOPYPART: IFCopyPart (); break;
  case A0_PERF_RECORD_ID_FCOPYPART: FCopyPart (); break;
  case A0_PERF_RECORD_ID_IFCOPY: IFCopy (); break;
  case A0_PERF_RECORD_ID_FCOPY: FCopy (); break;
  case A0_PERF_RECORD_ID_INIT_FREAD: InitFRead(); break;
  case A0_PERF_RECORD_ID_INIT_FWRITE: InitFWrite(); break;
  case A0_PERF_RECORD_ID_INIT_FCOPY: InitFCopy(); break;
  case A0_PERF_RECORD_ID_INIT_FLOCK: InitFLock(); break;
  case A0_PERF_RECORD_ID_INIT_FLOCKREAD: InitFLockRead(); break;
  case A0_PERF_RECORD_ID_INIT_FUNLOCK: InitFUnlock(); break;
  case A0_PERF_RECORD_ID_INIT_FUNLOCKWRITE: InitFUnlockWrite(); break;
  case A0_PERF_RECORD_ID_WRITECOPY: WriteCopy(); break;
  case A0_PERF_RECORD_ID_REPLYREAD: ReplyRead(); break;
  case A0_PERF_RECORD_ID_REPLYWRITE: ReplyWrite(); break;
  case A0_PERF_RECORD_ID_SETLOCKREAD: SetLockRead(); break;
  case A0_PERF_RECORD_ID_SETUNLOCKWRITE: SetUnlockWrite(); break;
  case A0_PERF_RECORD_ID_SETLOCKDMAREGION: SetLockDMARegion(); break;
  case A0_PERF_RECORD_ID_SETLOCKDMAREGIONPART: SetLockDMARegionPart(); break;
  case A0_PERF_RECORD_ID_LOCK: Lock(); break;
  case A0_PERF_RECORD_ID_UNLOCK: Unlock(); break;
  case A0_PERF_RECORD_ID_ILOCK: ILock(); break;
  case A0_PERF_RECORD_ID_IUNLOCK: IUnlock(); break;
  case A0_PERF_RECORD_ID_IFREADPARTLOCK: IFReadPartLock(); break;
  case A0_PERF_RECORD_ID_FREADPARTLOCK: FReadPartLock(); break;
  case A0_PERF_RECORD_ID_FWRITEUNLOCK: FWriteUnlock(); break;
  case A0_PERF_RECORD_ID_IFWRITEUNLOCK: IFWriteUnlock(); break;
  }
*buffer = a0_perf_index;
return (event);
}

char * _a0_perf_decode_buffer (char * buf)
{
char *a0_perf_index,*last;
a0_perf_reader_buffer *buffer = (a0_perf_reader_buffer *) buf;
int size;
char * car;
/* Get the effective content size */
a0_perf_index =  buf;
a0_perf_get_integer (size);
/* Now decode the effective content */
a0_perf_index = (char *) & (buffer -> content);
last = a0_perf_index + size;
while (a0_perf_index < last)	a0_perf_decode_event (&a0_perf_index);
return (a0_perf_index);
}

int a0_perf_decode_file (char *file_name)
{
int file_size;
long nb_lus;
char *end, *content, *position, *a0_perf_index;

/* Check file : open and get size */
fin = fopen (file_name,"r");
if (fin == NULL) 
   {
   fprintf (stderr,"Cannot read %s trace file\n",file_name);
   return(1);
   }
/* Compute file size */
/* Goto end of file */
fseek (fin,0,2);
file_size = ftell(fin);
/* Return to begin of the file */
fseek (fin,0,0);

if (file_size == -1) 
   {
   fprintf (stderr,"Cannot get the trace file size and infos\n"); 
   return(1);
   }
/* Read the whole file in memory with a single malloc */
content = malloc(file_size);
if (content == NULL)
  {
  fprintf (stderr,"Not enough memory to process the trace file\n");
  return(1);
  }
nb_lus = read(fileno(fin),content,file_size);
if (nb_lus != file_size)
  {
  fprintf (stderr,"Error : cannot read file content\n");
  return(1);
  }
fclose (fin);
end = content + file_size;
position = content;

/* Check trace versus redear releases */
if (strcmp (position,reader_release))
  fprintf (stderr,"Trace generator release %s\n Trace Reader release %s\n",
           position,reader_release);

/* Skip Release Sring */
position += A0_PERF_RELEASE_SIZE;

/* Read Nodecount */
a0_perf_index = position;
a0_perf_get_integer(a0_nodecount);


if ((check_nb_files) && (nb_files != a0_nodecount))
  {fprintf(stderr,"Nodecount = %d with %d trace files args\n",
   a0_nodecount,nb_files); return (1);}
  
/* Jump to Node record and read Node */
position += A0_NODE_SIZE;
a0_perf_index = position;
a0_perf_get_integer(a0_node);

/* Check node unicity */

if ((check_tab[a0_node]).is_processed)
  {
  fprintf (stderr, "Files %s and %s have the same node number\n",
           file_name,(check_tab[a0_node]).file_name);
  return (1);
  }
(check_tab[a0_node]).file_name = file_name;
(check_tab[a0_node]).is_processed = 1;

/* Read Initial_time */
position += A0_NODE_SIZE;
a0_perf_index = position;
{
int sec,usec;
a0_perf_get_integer (sec);
a0_perf_get_integer (usec);
a0_initial_time = (float) sec + 0.000001 * (float) usec;
}

/* No clock file to set time reference ? */
/* ==> use initial time of the first file, (usually node 0) */

if (!time_reference_is_defined)
  {
  time_reference = a0_initial_time;
  time_reference_is_defined = 1;
  }

/* Remove time_reference and offset from initial_time */
a0_initial_time -= (time_reference + check_tab [a0_node].offset);

/* Jump to the first buffer/block */
position += A0_PERF_TIME_SIZE;
a0_perf_index = position;

/* Decode buffers */
while (position <end)
  position = _a0_perf_decode_buffer (position);

/* free memory */
free (content);

return (0);
}


void a0_perf_write_header ()
{
int step;
a0_define_events = 1;
for (a0_source = 0;a0_source<=1;a0_source++)
  {
  SetLimits ();
  Abort ();
  BroadcastCondition ();
  ClearBuffer ();
  DetachSlave ();
  DisposeBuffer ();
  DisposeCondition ();
  DisposeMutex ();
  DisposePort ();
  DisposeSemaphore ();
  ExitThread ();
  GetRequestData ();
  GetThreadPriority ();
  GetThreadScheduling ();
  IReceive ();
  IStartRemoteUrgent ();
  IStartRemoteThread ();
  ISend ();
  ISendBuffer ();
  Init ();
  InitCommit ();
  IReceiveBuffer ();
  JoinSlave ();
  LockMutex ();
  NewBuffer();
  NewCondition ();
  NewMutex ();
  NewPort ();
  NewService ();
  NewSemaphore ();
  NewSlave ();
  PSemaphore ();
  Pack();
  Probe ();
  Receive ();
  ReceiveBuffer ();
  StartRemoteUrgent ();
  StartRemoteThread ();
  Send ();
  SendBuffer ();
  SetDefaultPriority ();
/*
  SetDefaultScheduling ();
*/
  SetDefaultStack ();
  SetThreadPriority ();
  SetThreadScheduling ();
  SignalCondition ();
  Terminate ();
  TestRequest ();
  TimedWaitCondition ();
  TryPSemaphore ();
  UnlockMutex ();
  Unpack ();
  UserEvent ();
  VSemaphore ();
  WaitCondition ();
  WaitRequest ();
  YieldThread ();
  IProbe ();
  IReceiveAny ();
  IReceiveBufferAny ();
  NewContext ();
  GetSelfContext ();
  SetSelfContext ();
  BeginUrgentService ();
  EndOfUserThread ();
  EndUrgentService ();
  IdentifyEvent();
  IdentifyFile ();
  InitUserThreadTraceBuffer ();
  BeginThreadService ();
  WaitTraceBuffer ();
  CompletedRequest (A0_PERF_RECORD_ID_COMPLETED_SEND);
  CompletedRequest (A0_PERF_RECORD_ID_COMPLETED_RECV);
  CompletedRequest (A0_PERF_RECORD_ID_COMPLETED_PROBE);
  CompletedRequest (A0_PERF_RECORD_ID_COMPLETED_OTHER);
  IdentifyFormat ();
  GetFormatSize();
  GetFormatOffset();
  DisposeFormat();
  NewMapFormat();
  SetMapField();
  NewFunctionFormat ();
  WaitAnyRequest ();
  TestAnyRequest ();
  IFPut ();
  FPut ();
  IFGet ();
  FGet ();
  Flush ();
  NewPipe ();
  DisposePipe ();
  UseFormats ();
  IFSend ();
  FSend ();
  IFReceive ();
  IFReceiveAny ();
  FReceive ();
  FPack ();
  FUnpack ();
  NewDmaRegion ();
  IFReadPart ();
  FReadPart ();
  IFRead ();
  FRead ();
  IFWritePart ();
  FWritePart ();
  IFWrite ();
  FWrite ();
  IFCopyPart ();
  FCopyPart ();
  IFCopy ();
  FCopy ();
  SetLockRead();
  SetUnlockWrite();
  SetLockDMARegion();
  SetLockDMARegionPart();
  Lock();
  ILock();
  Unlock();
  IUnlock();
  IFReadPartLock();
  FReadPartLock();
  FWriteUnlock();
  IFWriteUnlock();
#ifdef INTERNAL_FMEM
  InitFRead ();
  InitFWrite ();
  InitFCopy ();
  InitFLock ();
  InitFLockRead ();
  InitFUnlock ();
  InitFUnlockWrite ();
  WriteCopy ();
  ReplyRead ();
  ReplyWrite ();
#endif /* INTERNAL_FMEM */
  }
a0_define_events = 0;
a0_source = 0;
}

void a0_perf_init_min_max ()
{
min_time = + 1000000000000.0;
max_time = - 1000000000000.0;
}

void a0_perf_write_min_max ()
{
SetLimits ();
}
