#include "a0_perf_decode.h"
#include <stdio.h>
#include <stdlib.h>
#include "a0_perf_floor_sqrt.h"
#include <ath0.h>


char *content;				/* in memory adjust input file content */
char *current_number, *next_number;	/* positions within the content buffer */

void *memory_alloc (int size)
{
char *mem;
mem = malloc (size);
if (mem == NULL) 
   {
   fprintf (stderr,"Fatal error : cannot allocate memory \n");
   }
return (mem);
}

void read_file_content (char *content)
{
char  *buffer_ptr=content;
int c; 
int separator;
int comment;
/* Start : not within a comment, skip trealing separator */
buffer_ptr = content;
separator = 1;
comment = 0;

while ((c = getc(adjust_input_file)) != EOF)
  {
  switch (c)
    {
    case '@'  :
    case '#'  : comment = 1; break;     /* begin or continue comment */

    case '\n' : if (comment) {comment = 0; break;} /* end comment */
                /* If not within a comment, treat as any separator */

    case '$'  :
    case ' '  :
    case '\t' :
    case ','  :
    case ';'  :
    case '{'  :
    case '}'  : if (!comment && !separator)
                   {
                   separator = 1;
                   *buffer_ptr ++ = ' '; /* A single space replace any  */
                   }                      /* number of separators   */
                break;

    default   : if (!comment) 
                    {
                    *buffer_ptr++ = c; 
                    separator = 0;
                    }
    }
  }
}

double read_double (char *message,int node,double *value)
{
  *value = strtod (current_number,&next_number);
  if (next_number == current_number)
    {
    fprintf (stderr,"Adjust file content error (%s) node %d \n",message,node);
    return (1);
    }
  current_number= next_number;
  return (0);
}

/* Read the adjust input file and fill the file_check array */

int read_adjust_file ()
{
int file_size;
int node;
int node_number;
int integer_read;
int return_code = 0;

/* Compute file size */
fseek (adjust_input_file,0,2); 			/* Goto end of file */
file_size = ftell(adjust_input_file);
fseek (adjust_input_file,0,0); 			/* Return to begin of the file */
if (file_size == -1)
   {
   fprintf (stderr,"Cannot get the trace file size and infos\n");
   return_code = 1; goto close_file;
   }

/* Allocate the whole file in memory with a single malloc */
content = memory_alloc(file_size);
if (content == NULL) {return_code = 1 ; goto close_file;}

/* Read file in memory allocated buffer and skip comments */
read_file_content (content);

current_number = content;

/* Check number of nodes */
node_number = strtol  (current_number,&next_number,10);
if (next_number == current_number)
  {
  fprintf (stderr,"Adjust file content error (number of node missing)\n");
  return_code = 1; goto free_buffer;
  }
else
  current_number= next_number;

if (node_number != nb_files)
  {
  fprintf (stderr,"Fatal error on number of nodes : %d nodes in clock file for %d trace files\n",
	  node_number,nb_files);
  return_code = 1; goto free_buffer;
  }

/* Read initial_time */
time_reference = strtod (current_number,&next_number);
time_reference_is_defined = 1;

if (next_number == current_number)
  {
  fprintf (stderr,"Adjust file content error (time reference)\n");
  return_code = 1; goto free_buffer;
  }
else
  current_number= next_number;


/* fill adjust array */
for (node = 0; node <nb_files;node++)
  {
if (read_double ("Slope",node, &((check_tab + node) -> slope)))
    {return_code = 1; goto free_buffer;}
if (read_double ("Offset",node, &((check_tab + node) -> offset)))
    {return_code = 1; goto free_buffer;}
  }


/* free memory buffer and close input file */
free_buffer:
if (content != NULL) free(content);
close_file:
if (adjust_input_file != NULL) fclose (adjust_input_file);
return (return_code);
}
