
/* File a0_perf_internal_events.c                                           */
/* Philippe Waille 05/07/96                                                */
/***************************************************************************/

#define AKERNEL_SOURCE          /* To define the akWC constants */
#include "akwc.h"

#include "a0semap.h"
#include "a0thread.h"
#include "a0_perf_trace_buffer.h"
#include "a0_perf_internal_events.h"
#include "a0_perf_event_macros.h"
#include "a0_perf_event_codes.h"

/* Internal event : called within the newly created user thread */
/* within the harness for either slave or service thread */

a0tError a0_perf_init_user_thread_trace_buffer ()
{
unsigned long thread_id;
a0_perf_trace_buffer *buffer;
char *a0_perf_index;
a0_perf_declare_current_time;
buffer = a0_perf_get_free_trace_buffer ();
a0_perf_set_current_time;
a0_perf_index  = buffer -> content;
_a0GetSelfThreadId (&thread_id); 
_a0SetSelfContext (&a0_perf_Trace_buffer_context,buffer);
a0_perf_put_event_id(A0_PERF_RECORD_ID_INIT_TRACE_BUFFER);
a0_perf_put_integer(thread_id);
buffer -> index = a0_perf_index;
}

a0tError a0_perf_end_of_user_thread ()
{
a0_perf_begin_event;
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_END_OF_USER_THREAD);
a0_perf_put_index;
a0_perf_put_full_trace_buffer (trace_buffer);
}

a0tError a0_perf_start_remote_service (int service,unsigned long thread_id,
					a0tRequest *request)
{
a0tRequestData reqdata;
char *buf;
unsigned long serial_no;
int source;
a0_perf_begin_event;
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_START_REMOTE_SERVICE);
a0_perf_put_integer (service);
a0_perf_put_integer (thread_id);
_a0GetRequestSerialNo(request,&serial_no);
_a0GetRequestData (request,&reqdata);
a0_perf_put_integer (reqdata.source);
a0_perf_put_integer (serial_no);
a0_perf_put_index;
}

a0tError a0_perf_begin_remote_request (int service,a0tRequest *request)
{
a0tRequestData reqdata;
unsigned long serial_no;
int source;
unsigned long thread_id;
a0_perf_begin_event;
_a0GetSelfThreadId (&thread_id);
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_BEGIN_REMOTE_REQUEST);
a0_perf_put_integer (service);
_a0GetRequestSerialNo(request,&serial_no);
_a0GetRequestData (request,&reqdata);
a0_perf_put_integer (reqdata.source);
a0_perf_put_integer (serial_no);
a0_perf_put_index;
}

a0tError a0_perf_end_remote_request ()
{
a0_perf_begin_event;
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_END_REMOTE_REQUEST);
a0_perf_put_index;
}


a0tError a0_perf_internal_terminate ()
{
int i;
a0_perf_begin_event;
a0_perf_get_index;
a0_perf_put_event_id(A0_PERF_RECORD_ID_TERMINATE);
a0_perf_put_integer (a0_perf_nb_wait_buffer);
/* No put_index for Abort : */
/* Just copy index to the trace buffer */
/* No buffer overflow to check all buffers will be flushed anyway */
trace_buffer -> index = a0_perf_index;
a0_perf_flush_trace_buffers ();
}

a0tError a0_perf_completed_request (int type, unsigned long serial_no, 
				    int size, int source)
{
int write_origin=0;
a0_perf_begin_event;
a0_perf_get_index;
switch (type)
  {
  case akWCRECV :
    a0_perf_put_event_id(A0_PERF_RECORD_ID_COMPLETED_RECV);
    write_origin = 1;
    break;
  case akWCSEND :
    a0_perf_put_event_id(A0_PERF_RECORD_ID_COMPLETED_SEND);
    break;
  case akWCPROBE :
    a0_perf_put_event_id(A0_PERF_RECORD_ID_COMPLETED_PROBE);
    break;
  case akWCOTHER :
    a0_perf_put_event_id(A0_PERF_RECORD_ID_COMPLETED_OTHER);
    break;
  }
a0_perf_put_integer (serial_no);
a0_perf_put_integer (size);
if (write_origin) a0_perf_put_integer (source);
a0_perf_put_index;
}

