#ifndef PERF_TIMING_H
#define PERF_TIMING_H



#ifdef GET_TIME_OF_DAY

#include <sys/times.h>

#define a0_perf_declare_time_variable(var)	struct timeval var

#define a0_perf_get_time(var)			{\
						gettimeofday(&var,NULL);\
						}
#define a0_perf_put_initial_time	{\
			a0_perf_put_integer (a0_perf_global_time.tv_sec);\
			a0_perf_put_integer (a0_perf_global_time.tv_usec);\
			}

#define a0_perf_put_current_time		{\
			current_time.tv_sec -= a0_perf_global_time.tv_sec;\
			current_time.tv_usec -= a0_perf_global_time.tv_usec;\
			if (current_time.tv_usec <0)\
			  {\
			   current_time.tv_usec += 1000000;\
			   current_time.tv_sec --;\
			  }\
			a0_perf_put_integer (current_time.tv_sec);\
			a0_perf_put_integer (current_time.tv_usec);\
			}

#define a0_perf_time_to_double(t) (0.000001 * t.tv_usec + t.tv_sec)

#else

/* include <math.h> fails on Solaris  : redeclaration of `enum fp_direction_type' */
/* and I only use my own sqrt and floor prototype */

#include "a0_perf_floor_sqrt.h"

#define a0_perf_declare_time_variable(var)	double var


#define a0_perf_get_time(var)			{\
						var = MPI_Wtime();\
						}

#define a0_perf_put_initial_time			{\
			unsigned int  sec, usec;\
			a0_perf_declare_time_variable(tmp);\
			tmp = floor (a0_perf_global_time);\
			sec = (unsigned int) tmp;\
                        tmp = (a0_perf_global_time - tmp) * 1000000;\
                        usec = (unsigned int) tmp;\
			a0_perf_put_integer (sec);\
			a0_perf_put_integer (usec);\
			}
			

#define a0_perf_put_current_time		{\
			unsigned long  sec, usec;\
			a0_perf_declare_time_variable(tmp);\
			current_time -= a0_perf_global_time;\
			tmp = floor (current_time);\
			sec = (unsigned long) tmp;\
			tmp = (current_time - tmp) * 1000000;\
			usec = (unsigned long) tmp;\
			a0_perf_put_integer (sec);\
			a0_perf_put_integer (usec);\
			}

#define a0_perf_time_to_double(t) t

#endif
		


#define a0_perf_declare_current_time	\
			a0_perf_declare_time_variable (current_time)

#define a0_perf_set_current_time	\
			a0_perf_get_time (current_time)

#define a0_perf_set_initial_time	\
			a0_perf_get_time (a0_perf_global_time)

#define a0_perf_set_time(var)	a0_perf_get_time (var)


extern a0_perf_declare_time_variable (a0_perf_global_time);



#endif
