/*
 *
 * a0condvar.c : condition 
 *
 *
 * by Thierry Gautier 
 * Apache Project, IMAG, Grenoble, France
 * Feb 21, 1998
 *
 */

#include <akernel.h>
#include <errno.h>

#define A0_SOURCE
#include "a0thread.h"
#include "a0condvar.h"


/* debugging macro */
#define a0Debug_on(a) printf a
#define a0Debug(a)


/*************************************************************************
	_a0NewCondition - creates a new condition
*************************************************************************/

extern a0tError _a0NewCondition(a0tCondition *cond) {

  /* any errors here are fatal */
  if (akpthread_cond_init(cond, akPTHREAD_CONDATTR_DEFAULT_NP) !=0)
	_a0Error(A0ErrCondInit, "thread");
  return A0ErrOk;
}


/*************************************************************************
	_a0DisposeCondition - disposes a condition
*************************************************************************/

extern a0tError _a0DisposeCondition(a0tCondition *cond) {

  /* any errors here are fatal */
  int r;
  if ((r = akpthread_cond_destroy(cond)) !=0)
    return (r==EINVAL) ? A0ErrInvalid : A0ErrLockedMutex;

  return A0ErrOk;
}


/*************************************************************************
	_a0SignalCondition - signal a condition
*************************************************************************/

a0tError _a0SignalCondition(a0tCondition *cond) {

  /* any errors here are fatal */
  if (akpthread_cond_signal(cond) !=0)
	_a0Error(A0ErrCondSig, "thread");
  return A0ErrOk;
}


/*************************************************************************
	_a0BroadcastCondition - broadcast a condition  
*************************************************************************/

a0tError _a0BroadcastCondition(a0tCondition *cond) {

  /* any errors here are fatal */
  if (akpthread_cond_broadcast(cond) !=0)
	_a0Error(A0ErrCondSig, "but it's into broadcast");
  return A0ErrOk;
}


/*************************************************************************
	_a0WaitCondition - humm ???
*************************************************************************/

a0tError _a0WaitCondition(a0tCondition *cond, a0tMutex* mutex) {

  /* any errors here are fatal */
  int r;
  a0ThreadsBlockedLock++;
  if ( (r= akpthread_cond_wait(cond, mutex)) !=0)
    return (r==EINVAL) ? A0ErrInvalid : A0ErrCondWait;
  a0ThreadsBlockedLock--;

  return A0ErrOk;
}

/*************************************************************************
	_a0TimedwaitCondition - humm ???
*************************************************************************/

a0tError _a0TimedWaitCondition
   (a0tCondition *cond, a0tMutex *mutex, const struct timespec *abstime) {

  /* any errors here are fatal */
  int r;
  a0ThreadsBlockedLock++;
  if ( (r=akpthread_cond_timedwait(cond, mutex, (struct timespec *)abstime)) !=0)
    return (r ==ETIMEDOUT) ?  A0ErrTimedOut : A0ErrCondWait;
  a0ThreadsBlockedLock--;
}
