/*
 *
 * _a0pt2pt.h : point to point communications for Athapascan 0
 *
 * by Ilan Ginzburg
 * Apache Project, IMAG, Grenoble, France
 * May 31, 1996
 *
 */


/* Eliminates multiple includes of this file */
#ifndef _A0PT2PT_H
#define _A0PT2PT_H

#include <akernel.h>
#include "a0buffer.h"
#include "a0ports.h"

#define A0NoneCompleted akNOCOMPLETED

/* request used to get information on a terminated or pending send/recv */
typedef aktHandle a0tRequest;	/* akernel handle */

/* Request data structure. result of a0GetRequestData, the fields are
   known by the user. */
typedef struct _a0tRequestData {
  int source;		/* origin of the message */
  int size;		/* number of bytes arrived */
#ifdef A0_TRACE
  unsigned long serial_no; /* serial number of the request */
#endif
} a0tRequestData;


/* Waits for a request to terminate. */
extern a0tError _a0WaitRequest(a0tRequest *req);

/* Tests for the termination of a request. */
extern a0tError _a0TestRequest(a0tRequest *req, int *completed);

extern a0tError _a0WaitAnyRequest(a0tRequest *reqtab[], int size,
					int *which);

extern a0tError _a0TestAnyRequest(a0tRequest *reqtab[], int size,
					int *which);


/* Non-blocking probe of a port. tag is a USER tag. */
extern a0tError _a0IProbe(a0tPort *port, int source, int tag, a0tRequest *req);

/* Blocking probe of a port. tag is a USER tag. */
extern a0tError _a0Probe(a0tPort *port, int source, int tag, a0tRequest *req);


/* Get info from a0tRequest */
extern int _a0GetRequestData(a0tRequest *req, a0tRequestData *rd);


/* Nonblocking send of a buffer */
extern a0tError _a0ISendBuffer(a0tPort *port, int dest, int tag,
                                a0tRequest *request, a0tBuffer *buffer);

/* Performs a blocking send of a buffer */
extern a0tError _a0SendBuffer(a0tPort *port, int dest, int tag,
                                a0tRequest *request, a0tBuffer *buffer);

/* Performs a non-blocking send of a (datatype) message */
extern a0tError _a0ISend(a0tPort *port, int dest, int tag,
                                a0tRequest *request, a0tDatatype datatype,
                                void *address, int count);

/* Performs a blocking send of a (datatype) message.  */
extern a0tError _a0Send(a0tPort *port, int dest, int tag,
                                a0tRequest *request, a0tDatatype datatype,
                                void *address, int count);

/* Performs a nonblocking receive in a buffer */
extern a0tError _a0IReceiveBuffer(a0tPort *port, int source, int tag,
                                a0tRequest *request, a0tBuffer *buffer);

/* Performs a blocking receive in a buffer */
extern a0tError _a0ReceiveBuffer(a0tPort *port, int source, int tag,
                                a0tRequest *request, a0tBuffer *buffer);

/* Performs a nonblocking receive of a (datatype) message */
extern a0tError _a0IReceive(a0tPort *port, int source, int tag,
                                a0tRequest *request, a0tDatatype datatype,
                                void *address, int count);

/* Performs a blocking receive of a (datatype) message */
extern a0tError _a0Receive(a0tPort *port, int source, int tag,
                                a0tRequest *request, a0tDatatype datatype,
                                void *address, int count);


#ifdef A0_TRACE
/* Get serial number from a request */
#define _a0GetRequestSerialNo(rq,sn) (*(sn) = (rq)->serial_no)
#else
#define _a0GetRequestSerialNo(rq,sn) (*(sn)=0)
#endif /* A0_TRACE */


#ifdef A0_SOURCE
/***************************************************************************
 * Defines only used for compiling Athapascan0
 ***************************************************************************/


#define A0CannotAny 0
#define A0CanAny 1


extern a0tError _a0CheckNode(int node, int any);

/* Non-blocking probe of a port. tag is a USER tag. use callback */
extern a0tError _a0IProbeCallback(a0tPort *port, int source, int tag,
				a0tRequest *req,
				void (*callback)(), void *callback_par);

/* Performs a nonblocking receive in a buffer with a callback */
extern a0tError _a0IReceiveBufferCallback(a0tPort *port, int source, int tag,
                                a0tRequest *request, a0tBuffer *buffer,
				void (*callback)(), void *callback_par);

/* Performs a nonblocking receive of a (datatype) message */
extern a0tError _a0IReceiveCallback(a0tPort *port, int source, int tag,
                                a0tRequest *request, a0tDatatype datatype,
                                void *address, int count,
				void (*callback)(), void *callback_par);

/* Performs a nonblocking send in a buffer with a callback */
extern a0tError _a0ISendBufferCallback(a0tPort *port, int source, int tag,
                                a0tRequest *request, a0tBuffer *buffer,
				void (*callback)(), void *callback_par);

/* Performs a non-blocking send of a (datatype) message */
extern a0tError _a0ISendCallback(a0tPort *port, int dest, int tag,
                                a0tRequest *request, a0tDatatype datatype,
                                void *address, int count,
				void (*callback)(), void *callback_par);



#endif /* A0_SOURCE */

#endif /* _A0PT2PT_H */


