
/* Source file and line identification */

/* #define SOURCE_FILE_NAME  "user_file_name" before the */
/* #include <ath0.h>  triggers source line identification */


/* Source file name string is too long to be put in the trace file */
/* in each event record. File name is stored in a0perf_source_file_name */
/* whose address identify the source file and each event record this address */
/* instead of the string. As a result, a test is added to dump the content */
/* of a0perf_source_file_name on the first event record belonging to that */
/* file. This test is performed by the  _a0_source_perf_xxx  functions */
/* A null string means the source file name has already been dumped */

#ifndef ATH0_PERF_MACROS_H
#define ATH0_PERF_MACROS_H

#ifdef SOURCE_FILE_NAME
static char a0perf_source_file_name[256] = SOURCE_FILE_NAME; 
#define file	a0perf_source_file_name

/* Macros for source file and line identification */
/* Useful only if source file name available */

#define _a0_sperf_NewBuffer(a,b,c) \
	_a0_source_perf_NewBuffer(file,__LINE__,a,b,c)
#define _a0_sperf_DisposeBuffer(a) \
	_a0_source_perf_DisposeBuffer(file,__LINE__,a)
#define _a0_sperf_ClearBuffer(a) \
	_a0_source_perf_ClearBuffer(file,__LINE__,a)
#define _a0_sperf_Pack(a,b,c,d) \
        _a0_source_perf_Pack(file,__LINE__,a,b,c,d)
#define _a0_sperf_Unpack(a,b,c,d) \
        _a0_source_perf_Unpack(file,__LINE__,a,b,c,d)
#define _a0_sperf_Init(a,b) \
        _a0_source_perf_Init(file,__LINE__,a,b)
#define _a0_sperf_InitCommit() \
        _a0_source_perf_InitCommit(file,__LINE__)
#define _a0_sperf_Terminate() \
        _a0_source_perf_Terminate(file,__LINE__)
#define _a0_sperf_Abort(a) \
        _a0_source_perf_Abort(file,__LINE__,a)
#define _a0_sperf_NewPort(a,b,c) \
        _a0_source_perf_NewPort(file,__LINE__,a,b,c)
#define _a0_sperf_DisposePort(a) \
        _a0_source_perf_DisposePort(file,__LINE__,a)
#define _a0_sperf_WaitRequest(a) \
        _a0_source_perf_WaitRequest(file,__LINE__,a)
#define _a0_sperf_WaitAnyRequest(a,b,c) \
        _a0_source_perf_WaitAnyRequest(file,__LINE__,a,b,c)
#define _a0_sperf_TestRequest(a,b) \
        _a0_source_perf_TestRequest(file,__LINE__,a,b)
#define _a0_sperf_TestAnyRequest(a,b,c) \
        _a0_source_perf_TestAnyRequest(file,__LINE__,a,b,c)
#define _a0_sperf_IProbe(a,b,c,d) \
        _a0_source_perf_IProbe(file,__LINE__,a,b,c,d)
#define _a0_sperf_Probe(a,b,c,d) \
        _a0_source_perf_Probe(file,__LINE__,a,b,c,d)
#define _a0_sperf_ISendBuffer(a,b,c,d,e) \
        _a0_source_perf_ISendBuffer(file,__LINE__,a,b,c,d,e)
#define _a0_sperf_SendBuffer(a,b,c,d,e) \
        _a0_source_perf_SendBuffer(file,__LINE__,a,b,c,d,e)
#define _a0_sperf_ISend(a,b,c,d,e,f,g) \
        _a0_source_perf_ISend(file,__LINE__,a,b,c,d,e,f,g)
#define _a0_sperf_Send(a,b,c,d,e,f,g) \
        _a0_source_perf_Send(file,__LINE__,a,b,c,d,e,f,g)
#define _a0_sperf_IReceiveBuffer(a,b,c,d,e) \
        _a0_source_perf_IReceiveBuffer(file,__LINE__,a,b,c,d,e)
#define _a0_sperf_ReceiveBuffer(a,b,c,d,e) \
        _a0_source_perf_ReceiveBuffer(file,__LINE__,a,b,c,d,e)
#define _a0_sperf_IReceive(a,b,c,d,e,f,g) \
        _a0_source_perf_IReceive(file,__LINE__,a,b,c,d,e,f,g)
#define _a0_sperf_Receive(a,b,c,d,e,f,g) \
        _a0_source_perf_Receive(file,__LINE__,a,b,c,d,e,f,g)
#define _a0_sperf_NewService(a,b,c,d,e) \
       _a0_source_perf_NewService(file,__LINE__,a,b,c,d,e)
#define _a0_sperf_StartRemoteThread(a,b,c,d,e,f,g) \
       _a0_source_perf_StartRemoteThread(file,__LINE__,a,b,c,d,e,f,g)
#define _a0_sperf_IStartRemoteThread(a,b,c,d,e,f,g) \
      _a0_source_perf_IStartRemoteThread(file,__LINE__,a,b,c,d,e,f,g)
#define _a0_sperf_IStartRemoteUrgent(a,b,c,d) \
        _a0_source_perf_IStartRemoteUrgent(file,__LINE__,a,b,c,d)
#define _a0_sperf_StartRemoteUrgent(a,b,c,d) \
        _a0_source_perf_StartRemoteUrgent(file,__LINE__,a,b,c,d)
#define _a0_sperf_YieldThread() \
        _a0_source_perf_YieldThread(file,__LINE__)
#define _a0_sperf_JoinSlave(a,b) \
        _a0_source_perf_JoinSlave(file,__LINE__,a,b)
#define _a0_sperf_DetachSlave(a) \
        _a0_source_perf_DetachSlave(file,__LINE__,a)
#define _a0_sperf_ExitThread(a) \
        _a0_source_perf_ExitThread(file,__LINE__,a)
#define _a0_sperf_NewSlave(a,b,c,d,e,f) \
        _a0_source_perf_NewSlave(file,__LINE__,a,b,c,d,e,f)
#define _a0_sperf_NewSemaphore(a,b) \
        _a0_source_perf_NewSemaphore(file,__LINE__,a,b)
#define _a0_sperf_DisposeSemaphore(a) \
        _a0_source_perf_DisposeSemaphore(file,__LINE__,a)
#define _a0_sperf_PSemaphore(a) \
        _a0_source_perf_PSemaphore(file,__LINE__,a)
#define _a0_sperf_TryPSemaphore(a,b) \
        _a0_source_perf_TryPSemaphore(file,__LINE__,a,b)
#define _a0_sperf_VSemaphore(a) \
        _a0_source_perf_VSemaphore(file,__LINE__,a)
#define _a0_sperf_NewMutex(a) \
        _a0_source_perf_NewMutex(file,__LINE__,a)
#define _a0_sperf_DisposeMutex(a) \
        _a0_source_perf_DisposeMutex(file,__LINE__,a)
#define _a0_sperf_LockMutex(a) \
        _a0_source_perf_LockMutex(file,__LINE__,a)
#define _a0_sperf_UnlockMutex(a) \
        _a0_source_perf_UnlockMutex(file,__LINE__,a)
#define _a0_sperf_UserEvent(a,b) \
        _a0_source_perf_UserEvent(file,__LINE__,a,b)
#define _a0_sperf_NewContext(a) \
	_a0_source_perf_NewContext(file,__LINE__,a)
#define _a0_sperf_GetSelfContext(a,b) \
	_a0_source_perf_GetSelfContext(file,__LINE__,a,b)
#define _a0_sperf_SetSelfContext(a,b) \
	_a0_source_perf_SetSelfContext(file,__LINE__,a,b)
#else /* SOURCE_FILE_NAME */

/* If source file name not defined, translate source perf calls */
/* into normal perf calls */

#define _a0_sperf_UserEvent		_a0_perf_UserEvent
#define _a0_sperf_Abort			_a0_perf_Abort
#define _a0_sperf_ClearBuffer		_a0_perf_ClearBuffer
#define _a0_sperf_DetachSlave		_a0_perf_DetachSlave
#define _a0_sperf_DisposeBuffer		_a0_perf_DisposeBuffer
#define _a0_sperf_DisposeMutex		_a0_perf_DisposeMutex
#define _a0_sperf_DisposePort		_a0_perf_DisposePort
#define _a0_sperf_DisposeSemaphore	_a0_perf_DisposeSemaphore
#define _a0_sperf_ExitThread		_a0_perf_ExitThread
#define _a0_sperf_GetRequestSerialNo	_a0_perf_GetRequestSerialNo
#define _a0_sperf_GetSelfThreadId	_a0_perf_GetSelfThreadId
#define _a0_sperf_GetThreadId		_a0_perf_GetThreadId
#define _a0_sperf_IReceive		_a0_perf_IReceive
#define _a0_sperf_IReceiveBuffer	_a0_perf_IReceiveBuffer
#define _a0_sperf_IStartRemoteUrgent	_a0_perf_IStartRemoteUrgent
#define _a0_sperf_IStartRemoteThread	_a0_perf_IStartRemoteThread
#define _a0_sperf_ISend			_a0_perf_ISend
#define _a0_sperf_ISendBuffer		_a0_perf_ISendBuffer
#define _a0_sperf_Init			_a0_perf_Init
#define _a0_sperf_InitCommit		_a0_perf_InitCommit
#define _a0_sperf_JoinSlave		_a0_perf_JoinSlave
#define _a0_sperf_LockMutex		_a0_perf_LockMutex
#define _a0_sperf_NewBuffer		_a0_perf_NewBuffer
#define _a0_sperf_NewMutex		_a0_perf_NewMutex
#define _a0_sperf_NewPort		_a0_perf_NewPort
#define _a0_sperf_NewService		_a0_perf_NewService
#define _a0_sperf_NewSemaphore		_a0_perf_NewSemaphore
#define _a0_sperf_NewSlave		_a0_perf_NewSlave
#define _a0_sperf_PSemaphore		_a0_perf_PSemaphore
#define _a0_sperf_Pack			_a0_perf_Pack
#define _a0_sperf_Receive		_a0_perf_Receive
#define _a0_sperf_ReceiveBuffer		_a0_perf_ReceiveBuffer
#define _a0_sperf_StartRemoteUrgent	_a0_perf_StartRemoteUrgent
#define _a0_sperf_StartRemoteThread	_a0_perf_StartRemoteThread
#define _a0_sperf_Send			_a0_perf_Send
#define _a0_sperf_SendBuffer		_a0_perf_SendBuffer
#define _a0_sperf_Terminate		_a0_perf_Terminate
#define _a0_sperf_TestRequest		_a0_perf_TestRequest
#define _a0_sperf_TestAnyRequest	_a0_perf_TestAnyRequest
#define _a0_sperf_TryPSemaphore		_a0_perf_TryPSemaphore
#define _a0_sperf_UnlockMutex		_a0_perf_UnlockMutex
#define _a0_sperf_Unpack		_a0_perf_Unpack
#define _a0_sperf_VSemaphore		_a0_perf_VSemaphore
#define _a0_sperf_WaitRequest		_a0_perf_WaitRequest
#define _a0_sperf_WaitAnyRequest	_a0_perf_WaitAnyRequest
#define _a0_sperf_YieldThread		_a0_perf_YieldThread
#define _a0_sperf_Probe			_a0_perf_Probe
#define _a0_sperf_IProbe		_a0_perf_IProbe
#define _a0_sperf_NewContext		_a0_perf_NewContext
#define _a0_sperf_GetSelfContext	_a0_perf_GetSelfContext
#define _a0_sperf_SetSelfContext	_a0_perf_SetSelfContext

#endif /* SOURCE_FILE_NAME */

/* User may choose to not trace UserEvent */
/* He may define a0UserEvent as nothing */
/* But he could also define it as _a0UserEvent */
/* By symetry with other events */

/* Threfore nullify _a0UserEvent */
#define _a0UserEvent

#endif /* ATH0_PERF_MACROS_H */

