/*

 * akthread.c: DCE threads functions
 *
 * by Marcelo Pasin
 * Apache Project, IMAG, Grenoble, France
 * Jun 26, 1996
 *
 */

#include <pthread.h>
#include <errno.h>

#define AKERNEL_SOURCE
#include "akthread.h"
#include "akversion.h"
#include "akerror.h"


#ifdef AK_DCE
/**********************************************************************
	This part is included for DCE THREADS
**********************************************************************/
static char _nothing;
#endif /* AK_DCE */


#ifdef AK_POSIX
/**********************************************************************
	This part is included for POSIX THREADS
**********************************************************************/
#include <time.h>
pthread_attr_t akPTHREAD_ATTR_DEFAULT_NP;
size_t akPTHREAD_STACK_DEFAULT_NP;

#if defined(A0_TSOL2) || defined(A0_TSGI6)
int akpthread_get_delta_np(struct timespec *delta,
			   struct timespec *abstime)
{
	int i;

	if (i = clock_gettime(CLOCK_REALTIME, abstime))
		return i;
	abstime->tv_sec += delta->tv_sec;
	abstime->tv_nsec += delta->tv_nsec;

	return 0;
}
#endif /* A0_TSOL2 || A0_TSGI6 */
#endif /* AK_POSIX */


#ifdef A0_TRACE
/* only compile if traces are used */
/* This varible count how many threads exist in the system */
static unsigned long akLogicalThreadIdCount = 0;

/* key to the specific context */
static akpthread_key_t akLogicalThreadIdKey;

/* mutex to protect the ++ of akLogicalThreadIdCount */
static akpthread_mutex_t akLogicalThreadIdMutex;

#ifdef A0_TMAR
size_t akPTHREAD_STACK_DEFAULT_NP;
#endif /* A0_TMAR */


/*************************************************************************
akTraceInit - Startup of traces
*************************************************************************/

extern int akTraceInit(void)
{
	/* create the logical thread id key */
	if (akpthread_key_create(&akLogicalThreadIdKey, 0))
		akError(akERRCONTEXT, "akTraceInit");

	/* create the to protect its access */
	if (akpthread_mutex_init(&akLogicalThreadIdMutex,
				 akPTHREAD_MUTEXATTR_DEFAULT_NP))
		akError(akERRMUTEXINIT, "create trace");
}


/*************************************************************************
akTraceNewThreadId - generate new thread id
*************************************************************************/

extern int akTraceNewThreadId(unsigned long *idp)
{
	void *ptr;

	if (akpthread_mutex_lock(&akLogicalThreadIdMutex))
		akError(akERRMUTEXLOCK, "akTraceInit");

	*idp = akLogicalThreadIdCount++;

	if (akpthread_mutex_unlock(&akLogicalThreadIdMutex))
		akError(akERRMUTEXUNLOCK, "akTraceInit");
}


/*************************************************************************
akTraceSetThreadId - register thread id
*************************************************************************/

extern int akTraceSetThreadId(unsigned long id)
{
	/* insert the logical thread id */
	if (akpthread_setspecific(akLogicalThreadIdKey, (void *) id))
		akError(akERRCONTEXT, "set trace");

	a0_perf_init_user_thread_trace_buffer();
}


/*************************************************************************
akTraceGetThreadId - get registered thread id
*************************************************************************/

extern int akTraceGetThreadId(unsigned long *idp)
{
	void *id;
	/* get the logical thread id */
	if (akpthread_getspecific_np(akLogicalThreadIdKey, &id))
		akError(akERRCONTEXT, "get trace");
	*idp = (unsigned long) id;
}
#endif /* A0_TRACE */
